@@ -128,6 +128,8 @@ class KateBuildView : public Kate::PluginView, public Kate::XMLGUIClient
         KUrl              m_make_dir;
         QStack<KUrl>      m_make_dir_stack;
         QRegExp           m_filenameDetector;
+        QRegExp           m_filenameDetectorIcpc;
+        bool              m_filenameDetectorGccWorked;
         QRegExp           m_newDirDetector;
         unsigned int      m_numErrors;
         unsigned int      m_numWarnings;
