@@ -23,28 +23,22 @@
     Supported languages are C++, JS, Python, CMake, Bash
 '''
 
+import kate
+import kate.gui
 import os
 import re
 import textwrap
-
 from PyQt4 import uic
 from PyQt4.QtGui import *
 from PyKDE4.kdecore import i18nc
-from PyKDE4.kdeui import KXMLGUIClient
 from PyKDE4.ktexteditor import KTextEditor
 
-import kate
-import kate.ui
-import kate.view
-
 from libkatepate.decorators import *
-from libkatepate import common, selection
-
+from libkatepate import common, ui, selection
 # text processing predicates
 from libkatepate import pred
 from libkatepate.pred import neg, all_of, any_of
 
-
 BLOCK_ANY_START_SEARCH_RE = re.compile('^\s*#\s*if.*$')
 BLOCK_START_SEARCH_RE = re.compile('^\s*#\s*if\s*([01])$')
 BLOCK_ELSE_SEARCH_RE = re.compile('^\s*#\s*else.*$')
@@ -53,7 +47,6 @@ BLOCK_ELSE_ENDIF_MATCH_RE = re.compile('^\s*#\s*(endif|else).*$')
 BLOCK_START_GET_COND_RE = re.compile('^\s*#\s*(if((n)?def)?)\s+(.*)\s*$')
 COMMENT_START_POS = 'commentStartPos'
 
-
 def buildIfEndifMap(document):
     '''
         Build a list of tuples (start, end, elseif, is_comment) for all #if/#elseif/#endif blocks
@@ -175,10 +168,11 @@ def processLine(line, commentCh):
     return (result, column + len(commentCh) + 1)
 
 
-@kate.action
+@kate.action(i18nc('@action:inmenu', 'Inline Comment'), shortcut='Alt+D', menu='Edit')
 @check_constraints
+@comment_char_must_be_known()
 @selection_mode(selection.NORMAL)
-def inline_comment():
+def commentar():
     ''' Append or align an inlined comment at position 60 for the current line or the selection.
 
         Move cursor to the start of a comment, if nothing has changed.
@@ -234,8 +228,10 @@ def inline_comment():
         view.setCursorPosition(pos)
 
 
-@kate.action
-def move_above():
+@kate.action(i18nc('@action:inmenu', 'Move Comment Above'), shortcut='Meta+Left')
+@check_constraints
+@comment_char_must_be_known()
+def moveAbove():
     '''Move an inlined comment before the current line w/ same indentation level'''
     document = kate.activeDocument()
     view = kate.activeView()
@@ -283,8 +279,10 @@ def move_above():
         document.endEditing()                               # End transaction
 
 
-@kate.action
-def move_inline():
+@kate.action(i18nc('@action:inmenu', 'Move Comment Inline'), shortcut='Meta+Right')
+@check_constraints
+@comment_char_must_be_known()
+def moveInline():
     ''' Move a comment at the current line as inline comment of the next line
         (if latter has no comment yet)
     '''
@@ -356,8 +354,14 @@ def move_inline():
         document.endEditing()                               # End transaction
 
 
-@kate.action
-def comment_block():
+@kate.action(
+    i18nc('@action:inmenu', 'Comment Block with <command>#if0</command>')
+  , shortcut='Meta+D'
+  , menu='Edit'
+  )
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'C')
+def commentBlock():
     '''Wrap selected text (or current line) into a #if0/#endif block'''
     view = kate.activeView()
 
@@ -384,8 +388,14 @@ def comment_block():
         document.endEditing()                               # End transaction
 
 
-@kate.action
-def toggle_block():
+@kate.action(
+    i18nc('@action:inmenu', 'Toggle <icode>#if0</icode>/<icode>#if1</icode> Block')
+  , shortcut='Meta+Shift+D'
+  , menu='Edit'
+  )
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'C')
+def toggleBlock():
     ''' Switch a current code block to ON(#if1) or OFF(#if0) state.
 
         Current means that cursor placed inside of it.
@@ -417,7 +427,7 @@ def toggle_block():
         document.insertLine(blocksList[idx][0], '#if ' + newValue)
         document.endEditing()                                   # End transaction
     else:
-        kate.ui.popup(
+        ui.popup(
             i18nc('@title:window', 'Alert')
           , i18nc(
                 '@info:tooltip'
@@ -427,8 +437,10 @@ def toggle_block():
           )
 
 
-@kate.action
-def remove_block():
+@kate.action(i18nc('@action:inmenu', 'Remove <icode>#if0</icode> Block'), shortcut='Meta+R', menu='Edit')
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'C')
+def removeBlock():
     ''' Remove a block of code commented with #if0 or #if1-#else'''
     document = kate.activeDocument()
     view = kate.activeView()
@@ -469,7 +481,7 @@ def remove_block():
                 document.removeLine(blocksList[idx][0])
         document.endEditing()                                   # End transaction
     else:
-        kate.ui.popup(
+        ui.popup(
             i18nc('@title:window', 'Alert')
           , i18nc(
                 '@info:tooltip'
@@ -479,8 +491,14 @@ def remove_block():
           )
 
 
-@kate.action
-def select_block():
+@kate.action(
+    i18nc('@action:inmenu', 'Select Current <icode>#if0</icode>/<icode>#if1</icode> Block')
+  , shortcut='Meta+S'
+  , menu='Edit'
+  )
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'C')
+def selectBlock():
     '''Set selection of a current (where cursor positioned) #if0/#endif block'''
     document = kate.activeDocument()
     view = kate.activeView()
@@ -495,7 +513,7 @@ def select_block():
         r = KTextEditor.Range(blocksList[idx][0], 0, blocksList[idx][1] + 1, 0)
         view.setSelection(r)
     else:
-        kate.ui.popup(
+        ui.popup(
             i18nc('@title:window', 'Alert')
           , i18nc(
                 '@info:tooltip'
@@ -602,8 +620,11 @@ def turnFromBlockComment():
         document.endEditing()                               # End transaction
 
 
-@kate.action
-def toggle_doxy_comment():
+@kate.action(i18nc('@action:inmenu', 'Transform Doxygen Comments'), shortcut='Meta+X', menu='Edit')
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'JavaScript')
+@has_selection(False)
+def toggleDoxyComment():
     ''' Turn block of '///' doxygen comments into
         /**
          *
@@ -652,7 +673,7 @@ def changeParagraphWidth(step):
 
     originRange, isBlock = getParagraphRange(doc, pos)
     if originRange.isEmpty():
-        kate.ui.popup(
+        ui.popup(
             i18nc('@title:window', 'Alert')
           , i18nc(
                 '@info:tooltip'
@@ -712,14 +733,18 @@ def changeParagraphWidth(step):
         doc.endEditing()                                    # End transaction
 
 
-@kate.action
-def shrink_paragraph():
+@kate.action(i18nc('@action:inmenu', 'Shrink Comment Paragraph'), shortcut='Meta+[', menu='Edit')
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'JavaScript')
+def shrinkParagraph():
     '''Shrink a text paragraph width, whithing a comment, around the current cursor position'''
     changeParagraphWidth(-1)
 
 
-@kate.action
-def extend_paragraph():
+@kate.action(i18nc('@action:inmenu', 'Extend Comment Paragraph'), shortcut='Meta+]', menu='Edit')
+@check_constraints
+@restrict_doc_type('C++', 'C++11', 'C++11/Qt4', 'JavaScript')
+def extendParagraph():
     '''Extend a text paragraph width, whithing a comment, around the current cursor position'''
     changeParagraphWidth(1)
 
@@ -739,7 +764,7 @@ class ConfigWidget(QWidget):
         super(ConfigWidget, self).__init__(parent)
 
         # Set up the user interface from Designer.
-        uic.loadUi(os.path.join(os.path.dirname(__file__), 'commentar.ui'), self)
+        uic.loadUi(os.path.join(os.path.dirname(__file__), 'commentar_config.ui'), self)
 
         self.reset();
 
@@ -776,52 +801,13 @@ class ConfigPage(kate.Kate.PluginConfigPage, QWidget):
 
 
 @kate.configPage(
-    i18nc('@item:inmenu', 'Commentar Plugin')
+    i18nc('@action:inmenu', 'Commentar Plugin')
   , i18nc('@title:group', 'Commentar Plugin Settings')
   , icon="preferences-other"
   )
 def commentarConfigPage(parent=None, name=None):
     return ConfigPage(parent, name)
 
-
-@kate.view.selectionChanged
-def toggleSelectionSensitiveActions(view):
-    doc_type = view.document().highlightingMode()
-    clnt = kate.getXmlGuiClient()
-    has_selection = view.selection()
-
-    if doc_type in ['C++', 'C++/Qt4', 'JavaScript'] and not has_selection:
-        clnt.stateChanged('doxygen_actions')
-    else:
-        clnt.stateChanged('doxygen_actions', KXMLGUIClient.StateReverse)
-
-    if common.isKnownCommentStyle(doc_type) and not has_selection:
-        clnt.stateChanged('comment_actions_no_selection')
-    else:
-        clnt.stateChanged('comment_actions_no_selection', KXMLGUIClient.StateReverse)
-
-
-
-@kate.viewChanged
-def toggleDocTypeSeisitiveActions():
-    view = kate.activeView()
-    if view is None:
-        return
-    doc_type = view.document().highlightingMode()
-    kate.kDebug('toggleDocTypeSeisitiveActions: doc_type={}'.format(doc_type))
-    clnt = kate.getXmlGuiClient()
-
-    if doc_type in ['C++', 'C++/Qt4', 'C']:
-        clnt.stateChanged('if0_actions')
-    else:
-        clnt.stateChanged('if0_actions', KXMLGUIClient.StateReverse)
-
-    if common.isKnownCommentStyle(doc_type):
-        clnt.stateChanged('comment_actions')
-    else:
-        clnt.stateChanged('comment_actions', KXMLGUIClient.StateReverse)
-
-
 @kate.init
 def init():
     # Set default value if not configured yet
