@@ -562,7 +562,7 @@ class SearchBar(QObject):
         cursor = kate.activeView().cursorPosition()
         currentLine = cursor.line()
         document = kate.activeDocument()
-        self.historyModel.add(document.localFilePath(), "arrow-right", document.line(currentLine), currentLine, cursor.column(), "{}:{}".format(document.documentName(), currentLine + 1))
+        self.historyModel.add(document.url().toLocalFile(), "arrow-right", document.line(currentLine), currentLine, cursor.column(), "{}:{}".format(document.documentName(), currentLine + 1))
         #
         # Navigate to the point in the file.
         #
@@ -734,5 +734,7 @@ def viewChanged():
     use kate.activeView() '''
     global completionModel
     if completionModel:
-        completionInterface = kate.activeView().codeCompletionInterface()
-        completionInterface.registerCompletionModel(completionModel)
+        view = kate.activeView()
+        if view:
+            completionInterface = view.codeCompletionInterface()
+            completionInterface.registerCompletionModel(completionModel)
