@@ -28,6 +28,7 @@
 
 #include <ktexteditor/variableinterface.h>
 #include <ktexteditor/movingrange.h>
+#include <ktexteditor/range.h>
 
 #include <kconfig.h>
 #include <kdialog.h>
@@ -58,6 +59,9 @@
 #include <kdebug.h>
 //END
 
+/// Amount of characters the document may have to enable automatic invocation (1MB)
+static const int autoInvocationMaxFilesize = 1000000;
+
 //BEGIN KateWordCompletionModel
 KateWordCompletionModel::KateWordCompletionModel( QObject *parent )
   : CodeCompletionModel2( parent ), m_automatic(false)
@@ -78,8 +82,10 @@ void KateWordCompletionModel::saveMatches( KTextEditor::View* view,
 
 QVariant KateWordCompletionModel::data(const QModelIndex& index, int role) const
 {
+  if( role == UnimportantItemRole )
+    return QVariant(true);
   if( role == InheritanceDepth )
-    return 10000; //Very high value, so the word-completion group and items are shown behind any other groups/items if there is multiple
+    return 10000;
 
   if( !index.parent().isValid() ) {
     //It is the group header
@@ -143,21 +149,28 @@ int KateWordCompletionModel::rowCount ( const QModelIndex & parent ) const
 bool KateWordCompletionModel::shouldStartCompletion(KTextEditor::View* view, const QString &insertedText, bool userInsertion, const KTextEditor::Cursor &position)
 {
     if (!userInsertion) return false;
-    if(insertedText.isEmpty())
+    if (insertedText.isEmpty())
         return false;
 
 
     KateView *v = qobject_cast<KateView*> (view);
 
-    QString text = view->document()->line(position.line()).left(position.column());
-    uint check=v->config()->wordCompletionMinimalWordLength();
-    if (check<=0) return true;
-    int start=text.length();
-    int end=text.length()-check;
-    if (end<0) return false;
-    for (int i=start-1;i>=end;i--) {
-      QChar c=text.at(i);
-      if (! (c.isLetter() || (c.isNumber()) || c=='_') ) return false;
+    if (view->document()->totalCharacters() > autoInvocationMaxFilesize) {
+      // Disable automatic invocation for files larger than 1MB (see benchmarks)
+      return false;
+    }
+
+    const QString& text = view->document()->line(position.line()).left(position.column());
+    const uint check = v->config()->wordCompletionMinimalWordLength();
+    // Start completion immediately if min. word size is zero
+    if (!check) return true;
+    // Otherwise, check if user has typed long enough text...
+    const int start = text.length();
+    const int end = start - check;
+    if (end < 0) return false;
+    for (int i = start - 1; i >= end; i--) {
+      const QChar c = text.at(i);
+      if (!(c.isLetter() || (c.isNumber()) || c=='_')) return false;
     }
 
     return true;
@@ -177,65 +190,42 @@ bool KateWordCompletionModel::shouldAbortCompletion(KTextEditor::View* view, con
 
 void KateWordCompletionModel::completionInvoked(KTextEditor::View* view, const KTextEditor::Range& range, InvocationType it)
 {
-  /**
-   * auto invoke...
-   */
-  m_automatic=false;
-  if (it==AutomaticInvocation) {
-      m_automatic=true;
-      KateView *v = qobject_cast<KateView*> (view);
-
-      if (range.columnWidth() >= v->config()->wordCompletionMinimalWordLength())
-        saveMatches( view, range );
-      else
-        m_matches.clear();
-
-      // done here...
-      return;
-  }
-
-  // normal case ;)
+  m_automatic = it == AutomaticInvocation;
   saveMatches( view, range );
 }
 
 
-// Scan throughout the entire document for possible completions,
-// ignoring any dublets
-const QStringList KateWordCompletionModel::allMatches( KTextEditor::View *view, const KTextEditor::Range &range ) const
+/**
+ * Scan throughout the entire document for possible completions,
+ * ignoring any dublets and words shorter than configured and/or
+ * reasonable minimum length.
+ */
+QStringList KateWordCompletionModel::allMatches( KTextEditor::View *view, const KTextEditor::Range &range ) const
 {
-  QStringList l;
-  QRegExp re( "\\b(" + view->document()->text( range ) + "\\w{1,})" );
-  QSet<QString> seen;
-
-  /**
-   * scan only a range of the document to not die for LARGE files
-   */
-  const int lineLimit = 4096;
-  int start = qMax(0, range.start().line() - lineLimit);
-  int end = qMin(view->document()->lines(), range.start().line() + lineLimit);
-  for (int i = start; i < end; ++i)
-  {
-    QString s = view->document()->line( i );
-    int pos = 0;
-    while ( pos >= 0 )
-    {
-      pos = re.indexIn( s, pos );
-      if ( pos >= 0 )
-      {
-        // typing in the middle of a word
-        if ( ! ( i == range.start().line() && pos == range.start().column() ) )
-        {
-          QString m = re.cap( 1 );
-          if ( ! seen.contains( m ) ) {
-            seen.insert( m );
-            l << m;
-          }
+  QSet<QString> result;
+  const int minWordSize = qMax(2, qobject_cast<KateView*>(view)->config()->wordCompletionMinimalWordLength());
+  const int lines = view->document()->lines();
+  for ( int line = 0; line < lines; line++ ) {
+    const QString& text = view->document()->line(line);
+    int wordBegin = 0;
+    int offset = 0;
+    const int end = text.size();
+    while ( offset < end ) {
+      const QChar c = text.at(offset);
+      // increment offset when at line end, so we take the last character too
+      if ( ( ! c.isLetterOrNumber() && c != '_' ) || (offset == end - 1 && offset++) ) {
+        if ( offset - wordBegin > minWordSize && ( line != range.end().line() || offset != range.end().column() ) ) {
+          result.insert(text.mid(wordBegin, offset - wordBegin));
         }
-        pos += re.matchedLength();
+        wordBegin = offset + 1;
       }
+      if ( c.isSpace() ) {
+        wordBegin = offset + 1;
+      }
+      offset += 1;
     }
   }
-  return l;
+  return result.values();
 }
 
 void KateWordCompletionModel::executeCompletionItem2(
@@ -301,7 +291,7 @@ KTextEditor::Range KateWordCompletionModel::completionRange(KTextEditor::View* v
   KTextEditor::Document *doc = view->document();
   while ( col > 0 )
   {
-    QChar c = ( doc->character( KTextEditor::Cursor( line, col-1 ) ) );
+    const QChar c = ( doc->character( KTextEditor::Cursor( line, col-1 ) ) );
     if ( c.isLetterOrNumber() || c.isMark() || c == '_' )
     {
       col--;
@@ -578,11 +568,8 @@ QString KateWordCompletionView::findLongestUnique( const QStringList &matches, i
 {
   QString partial = matches.first();
 
-  QStringListIterator it( matches );
-  QString current;
-  while ( it.hasNext() )
+  foreach ( const QString& current, matches )
   {
-    current = it.next();
     if ( !current.startsWith( partial ) )
     {
       while( partial.length() > lead )
@@ -601,13 +588,13 @@ QString KateWordCompletionView::findLongestUnique( const QStringList &matches, i
 }
 
 // Return the string to complete (the letters behind the cursor)
-const QString KateWordCompletionView::word() const
+QString KateWordCompletionView::word() const
 {
   return m_view->document()->text( range() );
 }
 
 // Return the range containing the word behind the cursor
-const KTextEditor::Range KateWordCompletionView::range() const
+KTextEditor::Range KateWordCompletionView::range() const
 {
   return m_dWCompletionModel->completionRange(m_view, m_view->cursorPosition());
 }
