@@ -249,11 +249,9 @@ public:
     /**
      * Returns the x position of cursor \p col on the line \p range.
      */
-    int cursorToX(const KateTextLayout& range, int col) const;
+    int cursorToX(const KateTextLayout& range, int col, bool returnPastLine = false) const;
     /// \overload
-    int cursorToX(const KateTextLayout& range, const KTextEditor::Cursor& pos) const;
-    /// \overload
-    int cursorToX(const KateTextLayout& range, const KTextEditor::Cursor& pos, bool returnPastLine) const;
+    int cursorToX(const KateTextLayout& range, const KTextEditor::Cursor& pos, bool returnPastLine = false) const;
 
     /**
      * Returns the real cursor which is occupied by the specified x value, or that closest to it.
@@ -334,6 +332,9 @@ public:
 
     void assignSelectionBrushesFromAttribute(QTextLayout::FormatRange& target, const KTextEditor::Attribute& attribute) const;
 
+    // update font height
+    void updateFontHeight ();
+    
     KateDocument *const m_doc;
     Kate::TextFolding &m_folding;
     KateView *const m_view;
