@@ -32,6 +32,7 @@
 #include "kateviglobal.h"
 #include "katevinormalmode.h"
 #include "katerenderer.h"
+#include "kateviemulatedcommandbar.h"
 #include "katecmd.h"
 
 #include <kdebug.h>
@@ -551,7 +552,7 @@ const QStringList &KateCommands::ViCommands::cmds()
   static QStringList l;
 
   if (l.isEmpty())
-  l << "nnoremap" << "nn" << "d" << "delete" << "j" << "c" << "change" << "<" << ">" << "y" << "yank" <<
+  l << mappingCommands() << "d" << "delete" << "j" << "c" << "change" << "<" << ">" << "y" << "yank" <<
        "ma" << "mark" << "k";
 
   return l;
@@ -582,11 +583,25 @@ bool KateCommands::ViCommands::exec(KTextEditor::View *view,
   QStringList args(_cmd.split( QRegExp("\\s+"), QString::SkipEmptyParts)) ;
   QString cmd ( args.takeFirst() );
 
+
   // ALL commands that takes no arguments.
-  if ( cmd == "nnoremap" || cmd == "nn" )
+  if (mappingCommands().contains(cmd))
   {
+    if (cmd.endsWith("unmap"))
+    {
+      if (args.count() == 1)
+      {
+        KateGlobal::self()->viInputModeGlobal()->removeMapping(modeForMapCommand(cmd), args.at(0));
+        return true;
+      }
+      else
+      {
+        msg = i18n("Missing argument. Usage: %1 <from>",  cmd );
+        return false;
+      }
+    }
     if ( args.count() == 1 ) {
-      msg = KateGlobal::self()->viInputModeGlobal()->getMapping( NormalMode, args.at( 0 ), true );
+      msg = KateGlobal::self()->viInputModeGlobal()->getMapping( modeForMapCommand(cmd), args.at( 0 ), true );
       if ( msg.isEmpty() ) {
         msg = i18n( "No mapping found for \"%1\"", args.at(0) );
         return false;
@@ -594,8 +609,8 @@ bool KateCommands::ViCommands::exec(KTextEditor::View *view,
         msg = i18n( "\"%1\" is mapped to \"%2\"", args.at( 0 ), msg );
       }
     } else if ( args.count() == 2 ) {
-      KateViModeBase::MappingRecursion mappingRecursion = (cmd == "nnoremap") ? KateViModeBase::NonRecursive : KateViModeBase::Recursive;
-      KateGlobal::self()->viInputModeGlobal()->addMapping( NormalMode, args.at( 0 ), args.at( 1 ), mappingRecursion);
+      KateViGlobal::MappingRecursion mappingRecursion = (isMapCommandRecursive(cmd)) ? KateViGlobal::Recursive : KateViGlobal::NonRecursive;
+      KateGlobal::self()->viInputModeGlobal()->addMapping( modeForMapCommand(cmd), args.at( 0 ), args.at( 1 ), mappingRecursion);
     } else {
       msg = i18n("Missing argument(s). Usage: %1 <from> [<to>]",  cmd );
       return false;
@@ -615,7 +630,6 @@ bool KateCommands::ViCommands::exec(KTextEditor::View *view,
     int count = 1;
     if (range.isValid()){
         count = qAbs(range.end().line() - range.start().line())+1;
-        if (cmd == "j") count-=1;
         v->setCursorPosition(KTextEditor::Cursor(qMin(range.start().line(),
                                                       range.end().line()),0));
     }
@@ -705,7 +719,7 @@ KCompletion *KateCommands::ViCommands::completionObject( KTextEditor::View *view
 
   if ( v && ( cmd == "nn" || cmd == "nnoremap" ) )
   {
-    QStringList l = KateGlobal::self()->viInputModeGlobal()->getMappings( NormalMode );
+    QStringList l = KateGlobal::self()->viInputModeGlobal()->getMappings( KateViGlobal::NormalModeMapping );
 
     KateCmdShellCompletion *co = new KateCmdShellCompletion();
     co->setItems( l );
@@ -714,6 +728,66 @@ KCompletion *KateCommands::ViCommands::completionObject( KTextEditor::View *view
   }
   return 0L;
 }
+
+const QStringList& KateCommands::ViCommands::mappingCommands()
+{
+  static QStringList mappingsCommands;
+  if (mappingsCommands.isEmpty())
+  {
+    mappingsCommands << "nmap"  << "nm"  << "noremap" << "nnoremap" << "nn" << "no"
+                     << "vmap" << "vm" << "vnoremap" << "vn"
+                     << "imap" << "im" << "inoremap" << "ino"
+                     << "cmap" << "cm" << "cnoremap" << "cno";
+
+    mappingsCommands << "nunmap" << "vunmap" << "iunmap" << "cunmap";
+  }
+  return mappingsCommands;
+}
+
+KateViGlobal::MappingMode KateCommands::ViCommands::modeForMapCommand(const QString& mapCommand)
+{
+  static QMap<QString, KateViGlobal::MappingMode> modeForMapCommand;
+  if (modeForMapCommand.isEmpty())
+  {
+    // Normal is the default.
+    modeForMapCommand.insert("vmap", KateViGlobal::VisualModeMapping);
+    modeForMapCommand.insert("vm", KateViGlobal::VisualModeMapping);
+    modeForMapCommand.insert("vnoremap", KateViGlobal::VisualModeMapping);
+    modeForMapCommand.insert("vn", KateViGlobal::VisualModeMapping);
+    modeForMapCommand.insert("imap", KateViGlobal::InsertModeMapping);
+    modeForMapCommand.insert("im", KateViGlobal::InsertModeMapping);
+    modeForMapCommand.insert("inoremap", KateViGlobal::InsertModeMapping);
+    modeForMapCommand.insert("ino", KateViGlobal::InsertModeMapping);
+    modeForMapCommand.insert("cmap", KateViGlobal::CommandModeMapping);
+    modeForMapCommand.insert("cm", KateViGlobal::CommandModeMapping);
+    modeForMapCommand.insert("cnoremap", KateViGlobal::CommandModeMapping);
+    modeForMapCommand.insert("cno", KateViGlobal::CommandModeMapping);
+
+    modeForMapCommand.insert("nunmap", KateViGlobal::NormalModeMapping);
+    modeForMapCommand.insert("vunmap", KateViGlobal::VisualModeMapping);
+    modeForMapCommand.insert("iunmap", KateViGlobal::InsertModeMapping);
+    modeForMapCommand.insert("cunmap", KateViGlobal::CommandModeMapping);
+  }
+  return modeForMapCommand[mapCommand];
+}
+
+bool KateCommands::ViCommands::isMapCommandRecursive(const QString& mapCommand)
+{
+  static QMap<QString, bool> isMapCommandRecursive;
+  {
+    isMapCommandRecursive.insert("nmap", true);
+    isMapCommandRecursive.insert("nm", true);
+    isMapCommandRecursive.insert("vmap", true);
+    isMapCommandRecursive.insert("vm", true);
+    isMapCommandRecursive.insert("imap", true);
+    isMapCommandRecursive.insert("im", true);
+    isMapCommandRecursive.insert("cmap", true);
+    isMapCommandRecursive.insert("cm", true);
+  }
+  return isMapCommandRecursive[mapCommand];
+}
+
+
 //END ViCommands
 
 // BEGIN AppCommands
@@ -796,19 +870,6 @@ bool KateCommands::AppCommands::help(KTextEditor::View *view, const QString &cmd
 //BEGIN SedReplace
 KateCommands::SedReplace* KateCommands::SedReplace::m_instance = 0;
 
-static void replace(QString &s, const QString &needle, const QString &with)
-{
-  int pos = 0;
-  while (1)
-  {
-    pos = s.indexOf(needle, pos);
-    if (pos == -1) break;
-    s.replace(pos, needle.length(), with);
-    pos += with.length();
-  }
-
-}
-
 static int backslashString(const QString &haystack, const QString &needle, int index)
 {
   int len = haystack.length();
@@ -856,115 +917,6 @@ static void exchangeAbbrevs(QString &str)
   }
 }
 
-int KateCommands::SedReplace::sedMagic( KateDocument *doc, int &line,
-                                        const QString &find, const QString &repOld, const QString &delim,
-                                        bool noCase, bool repeat,
-                                        int startcol, int endcol )
-{
-  Kate::TextLine ln = doc->kateTextLine( line );
-  if ( !ln || !ln->length() ) return 0;
-
-  // HANDLING "\n"s in PATTERN
-  // * Create a list of patterns, splitting PATTERN on (unescaped) "\n"
-  // * insert $s and ^s to match line ends/beginnings
-  // * When matching patterns after the first one, replace \N with the captured
-  //   text.
-  // * If all patterns in the list match sequential lines, there is a match, so
-  // * remove line/start to line + patterns.count()-1/patterns.last.length
-  // * handle captures by putting them in one list.
-  // * the existing insertion is fine, including the line calculation.
-
-  QStringList patterns(find.split( QRegExp("(^\\\\n|(?![^\\\\])\\\\n)"), QString::KeepEmptyParts));
-  if ( patterns.count() > 1 )
-  {
-    for ( int i = 0; i < patterns.count(); ++i )
-    {
-      if ( i < patterns.count() - 1 )
-        patterns[i].append("$");
-      if ( i )
-        patterns[i].prepend("^");
-
-       kDebug(13025) << "patterns[" << i << "] =" << patterns[i];
-    }
-  }
-
-  QRegExp matcher(patterns[0], noCase ?Qt::CaseSensitive:Qt::CaseInsensitive);
-
-  int matches = 0;
-
-  while ( (startcol = matcher.indexIn(ln->string(), startcol)) >= 0 )
-  {
-    const int len = matcher.matchedLength();
-
-    if ( endcol >= 0 && startcol + len > endcol )
-      break;
-
-    ++matches;
-
-
-    QString rep = repOld;
-
-    // now set the backreferences in the replacement
-    const QStringList backrefs = matcher.capturedTexts();
-    int refnum = 1;
-
-    QStringList::ConstIterator i = backrefs.begin();
-    ++i;
-
-    for (; i != backrefs.end(); ++i)
-    {
-      // I need to match "\\" or "", but not "\"
-      QString number = QString::number(refnum);
-
-      int index = 0;
-      while (index != -1)
-      {
-        index = backslashString(rep, number, index);
-        if (index >= 0)
-        {
-          rep.replace(index, 2, *i);
-          index += (*i).length();
-        }
-      }
-
-      ++refnum;
-    }
-
-    replace(rep, "\\\\", "\\");
-    replace(rep, "\\" + delim, delim);
-
-    doc->removeText( KTextEditor::Range (line, startcol, line, startcol + len) );
-    doc->insertText( KTextEditor::Cursor (line, startcol), rep );
-
-    // TODO if replace contains \n,
-    // change the line number and
-    // check for text that needs be searched behind the last inserted newline.
-    int lns = rep.count(QChar::fromLatin1('\n'));
-    if ( lns > 0 )
-    {
-      line += lns;
-
-      if ( doc->lineLength( line ) > 0 && ( endcol < 0 || endcol >= startcol + len ) )
-      {
-      // if ( endcol >= startcol + len )
-          endcol -= (startcol + len);
-          uint sc = rep.length() - rep.lastIndexOf('\n') - 1;
-        matches += sedMagic( doc, line, find, repOld, delim, noCase, repeat, sc, endcol );
-      }
-    }
-
-    if (!repeat) break;
-    startcol += rep.length();
-
-    // sanity check -- avoid infinite loops eg with %s,.*,,g ;)
-    int ll = ln->length();
-    if ( !ll || startcol > (ll - 1) )
-      break;
-  }
-
-  return matches;
-}
-
 bool KateCommands::SedReplace::exec (KTextEditor::View *view, const QString &cmd, QString &msg)
 {
   return exec(view, cmd, msg, KTextEditor::Range::invalid());
@@ -978,77 +930,218 @@ bool KateCommands::SedReplace::exec (class KTextEditor::View *view, const QStrin
     kDebug(13025) << "Range: " << r;
   }
 
+  int findBeginPos = -1;
+  int findEndPos = -1;
+  int replaceBeginPos = -1;
+  int replaceEndPos = -1;
+  QString delimiter;
+  if (!parse(cmd, delimiter, findBeginPos, findEndPos, replaceBeginPos, replaceEndPos))
+  {
+    return false;
+  }
+
+  const QString searchParamsString = cmd.mid(cmd.lastIndexOf(delimiter));
+  const bool noCase = searchParamsString.contains('i');
+  const bool repeat = searchParamsString.contains('g');
+  const bool interactive = searchParamsString.contains('c');
+
+  QString find = cmd.mid(findBeginPos, findEndPos - findBeginPos + 1);
+  kDebug(13025) << "SedReplace: find =" << find;
+
+  QString replace = cmd.mid(replaceBeginPos, replaceEndPos - replaceBeginPos + 1);
+  exchangeAbbrevs(replace);
+  kDebug(13025) << "SedReplace: replace =" << replace;
+
+  if (find.isEmpty())
+  {
+    // Nothing to do.
+    return true;
+  }
+
+  KateView *kateView = static_cast<KateView*>(view);
+  KateDocument *doc = kateView->doc();
+  if ( !doc ) return false;
+  // Only current line ...
+  int startLine = kateView->cursorPosition().line();
+  int endLine = kateView->cursorPosition().line();
+  // ... unless a range was provided.
+  if (r.isValid())
+  {
+    startLine = r.start().line();
+    endLine = r.end().line();
+  }
+
+  QSharedPointer<InteractiveSedReplacer> interactiveSedReplacer(new InteractiveSedReplacer(doc, find, replace, !noCase, !repeat, startLine, endLine));
+
+  if (interactive)
+  {
+    if (kateView->viInputMode() && KateViewConfig::global()->viInputModeEmulateCommandBar())
+    {
+      KateViEmulatedCommandBar *emulatedCommandBar = kateView->viModeEmulatedCommandBar();
+      emulatedCommandBar->startInteractiveSearchAndReplace(interactiveSedReplacer);
+      return true;
+    }
+    else
+    {
+      kDebug(13025) << "Interactive sedreplace is only currently supported with Vi mode plus Vi emulated command bar.";
+      return false;
+    }
+  }
+
+  kateView->setSearchPattern(find);
+
+  interactiveSedReplacer->replaceAllRemaining();
+  msg = interactiveSedReplacer->finalStatusReportMessage();
+
+  return true;
+}
+
+bool KateCommands::SedReplace::parse(const QString& sedReplaceString, QString& destDelim, int& destFindBeginPos, int& destFindEndPos, int& destReplaceBeginPos, int& destReplaceEndPos)
+{
   // valid delimiters are all non-word, non-space characters plus '_'
   QRegExp delim("^s\\s*([^\\w\\s]|_)");
-  if ( delim.indexIn( cmd ) < 0 ) return false;
-
-  bool noCase = cmd[cmd.length() - 1] == 'i' || cmd[cmd.length() - 2] == 'i';
-  bool repeat = cmd[cmd.length() - 1] == 'g' || cmd[cmd.length() - 2] == 'g';
+  if ( delim.indexIn( sedReplaceString ) < 0 ) return false;
 
   QString d = delim.cap(1);
   kDebug(13025) << "SedReplace: delimiter is '" << d << "'";
 
   QRegExp splitter( QString("^s\\s*") + d + "((?:[^\\\\\\" + d + "]|\\\\.)*)\\"
-      + d + "((?:[^\\\\\\" + d + "]|\\\\.)*)(\\" + d + "[ig]{0,2})?$" );
-  if (splitter.indexIn(cmd) < 0) return false;
+      + d + "((?:[^\\\\\\" + d + "]|\\\\.)*)(\\" + d + "[igc]{0,3})?$" );
+  if (splitter.indexIn(sedReplaceString) < 0) return false;
 
-  QString find = splitter.cap(1);
-  kDebug(13025) << "SedReplace: find =" << find;
+  const QString find = splitter.cap(1);
+  const QString replace = splitter.cap(2);
 
-  QString replace = splitter.cap(2);
-  exchangeAbbrevs(replace);
-  kDebug(13025) << "SedReplace: replace =" << replace;
-
-  if ( find.contains("\\n") )
-  {
-    // FIXME: make replacing newlines work
-    msg = i18n("Sorry, but Kate is not able to replace newlines, yet");
-    return false;
-  }
-
-  KateDocument *doc = static_cast<KateView*>(view)->doc();
-  if ( !doc ) return false;
-
-  static_cast<KateView*>(view)->setSearchPattern(find);
-  doc->editStart();
-
-  int replacementsDone = 0;
-  int linesTouched = 0;
-  int linesAdded = 0;
-
-  if (r.isValid()) { // given range
-    for (int line = r.start().line(); line <= r.end().line() + linesAdded; ++line) {
-      int temp = replacementsDone;
-      int r = sedMagic( doc, line, find, replace, d, !noCase, repeat );
-      replacementsDone += r;
-
-      // if we replaced the text with n newlines, we have n new lines to look at
-      if (replace.contains('\n')) {
-        linesAdded += r * replace.count('\n');
-      }
-
-      if (replacementsDone > temp) {
-        ++linesTouched;
-      }
-    }
-  } else { // current line
-    int line = view->cursorPosition().line();
-    replacementsDone += sedMagic(doc, line, find, replace, d, !noCase, repeat);
-    if (replacementsDone > 0) {
-      linesTouched = 1;
-    }
-  }
-
-  msg = i18ncp("%2 is the translation of the next message",
-               "1 replacement done on %2", "%1 replacements done on %2", replacementsDone,
-               i18ncp("substituted into the previous message",
-                      "1 line", "%1 lines", linesTouched));
-
-  doc->editEnd();
+  destDelim = d;
+  destFindBeginPos = splitter.pos(1);
+  destFindEndPos = splitter.pos(1) + find.length() - 1;
+  destReplaceBeginPos = splitter.pos(2);
+  destReplaceEndPos = splitter.pos(2) + replace.length() - 1;
 
   return true;
 }
 
+KateCommands::SedReplace::InteractiveSedReplacer::InteractiveSedReplacer(KateDocument* doc, const QString& findPattern, const QString& replacePattern, bool caseSensitive, bool onlyOnePerLine, int startLine, int endLine)
+    : m_findPattern(findPattern),
+      m_replacePattern(replacePattern),
+      m_onlyOnePerLine(onlyOnePerLine),
+      m_startLine(startLine),
+      m_endLine(endLine),
+      m_doc(doc),
+      m_regExpSearch(doc, caseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive),
+      m_numReplacementsDone(0),
+      m_numLinesTouched(0),
+      m_lastChangedLineNum(-1)
+{
+  m_currentSearchPos = Cursor(startLine, 0);
+}
+
+Range KateCommands::SedReplace::InteractiveSedReplacer::currentMatch()
+{
+  const Range currentMatch = fullCurrentMatch().first();
+  if (currentMatch.start().line() > m_endLine)
+  {
+    return Range::invalid();
+  }
+  return currentMatch;
+}
+
+void KateCommands::SedReplace::InteractiveSedReplacer::skipCurrentMatch()
+{
+  const Range currentMatch = this->currentMatch();
+  m_currentSearchPos = currentMatch.end();
+  if (m_onlyOnePerLine && currentMatch.start().line() == m_currentSearchPos.line())
+  {
+    m_currentSearchPos = Cursor(m_currentSearchPos.line() + 1, 0);
+  }
+}
+
+void KateCommands::SedReplace::InteractiveSedReplacer::replaceCurrentMatch()
+{
+  const Range currentMatch = this->currentMatch();
+  const QString currentMatchText = m_doc->text(currentMatch);
+  const QString replacementText = replacementTextForCurrentMatch();
+
+  m_doc->editBegin();
+  m_doc->removeText(currentMatch);
+  m_doc->insertText(currentMatch.start(), replacementText);
+  m_doc->editEnd();
+
+  // Begin next search from directly after replacement.
+  if (!replacementText.contains('\n'))
+  {
+    m_currentSearchPos = Cursor(currentMatch.start().line(), currentMatch.start().column() + replacementText.length());
+  }
+  else
+  {
+    m_currentSearchPos = Cursor(currentMatch.start().line() + replacementText.count('\n'),
+                                replacementText.length() - replacementText.lastIndexOf('\n') - 1);
+  }
+  if (m_onlyOnePerLine)
+  {
+    // Drop down to next line.
+    m_currentSearchPos = Cursor(m_currentSearchPos.line() + 1, 0);
+  }
+
+  // Adjust end line down by the number of new newlines just added, minus the number taken away.
+  m_endLine += replacementText.count('\n');
+  m_endLine -= currentMatchText.count('\n');
+
+  m_numReplacementsDone++;
+  if (m_lastChangedLineNum != currentMatch.start().line())
+  {
+    // Counting "swallowed" lines as being "touched".
+    m_numLinesTouched += currentMatchText.count('\n') + 1;
+  }
+  m_lastChangedLineNum = m_currentSearchPos.line();
+}
+
+void KateCommands::SedReplace::InteractiveSedReplacer::replaceAllRemaining()
+{
+  m_doc->editBegin();
+  while (currentMatch().isValid())
+  {
+    replaceCurrentMatch();
+  }
+  m_doc->editEnd();
+}
+
+QString KateCommands::SedReplace::InteractiveSedReplacer::currentMatchReplacementConfirmationMessage()
+{
+  return i18n("replace with %1?", replacementTextForCurrentMatch().replace('\n', "\\n"));
+}
+
+QString KateCommands::SedReplace::InteractiveSedReplacer::finalStatusReportMessage()
+{
+  return i18ncp("%2 is the translation of the next message",
+               "1 replacement done on %2", "%1 replacements done on %2", m_numReplacementsDone,
+               i18ncp("substituted into the previous message",
+                      "1 line", "%1 lines", m_numLinesTouched));
+
+}
+
+
+const QVector< Range > KateCommands::SedReplace::InteractiveSedReplacer::fullCurrentMatch()
+{
+  return m_regExpSearch.search(m_findPattern, Range(m_currentSearchPos, m_doc->documentEnd()));
+}
+
+QString KateCommands::SedReplace::InteractiveSedReplacer::replacementTextForCurrentMatch()
+{
+  const Range currentMatch = this->currentMatch();
+  const QVector<KTextEditor::Range> captureRanges = fullCurrentMatch();
+  QStringList captureTexts;
+  foreach(const Range& captureRange, captureRanges)
+  {
+    captureTexts << m_doc->text(captureRange);
+  }
+  const QString replacementText = m_regExpSearch.buildReplacement(m_replacePattern, captureTexts, 0);
+  return replacementText;
+
+}
+
+
+
 //END SedReplace
 
 //BEGIN Character
