@@ -413,6 +413,9 @@ void KateDocumentConfig::setTabWidth (int tabWidth)
   if (tabWidth < 1)
     return;
 
+  if (m_tabWidthSet && m_tabWidth == tabWidth)
+    return;
+
   configStart ();
 
   m_tabWidthSet = true;
@@ -434,6 +437,9 @@ void KateDocumentConfig::setIndentationWidth (int indentationWidth)
   if (indentationWidth < 1)
     return;
 
+  if (m_indentationWidthSet && m_indentationWidth == indentationWidth)
+    return;
+
   configStart ();
 
   m_indentationWidthSet = true;
@@ -452,6 +458,9 @@ const QString &KateDocumentConfig::indentationMode () const
 
 void KateDocumentConfig::setIndentationMode (const QString &indentationMode)
 {
+  if (m_indentationModeSet && m_indentationMode == indentationMode)
+    return;
+
   configStart ();
 
   m_indentationModeSet = true;
@@ -489,6 +498,9 @@ bool KateDocumentConfig::wordWrap () const
 
 void KateDocumentConfig::setWordWrap (bool on)
 {
+  if (m_wordWrapSet && m_wordWrap == on)
+    return;
+
   configStart ();
 
   m_wordWrapSet = true;
@@ -497,7 +509,7 @@ void KateDocumentConfig::setWordWrap (bool on)
   configEnd ();
 }
 
-unsigned int KateDocumentConfig::wordWrapAt () const
+int KateDocumentConfig::wordWrapAt () const
 {
   if (m_wordWrapAtSet || isGlobal())
     return m_wordWrapAt;
@@ -505,11 +517,14 @@ unsigned int KateDocumentConfig::wordWrapAt () const
   return s_global->wordWrapAt();
 }
 
-void KateDocumentConfig::setWordWrapAt (unsigned int col)
+void KateDocumentConfig::setWordWrapAt (int col)
 {
   if (col < 1)
     return;
 
+  if (m_wordWrapAtSet && m_wordWrapAt == col)
+    return;
+
   configStart ();
 
   m_wordWrapAtSet = true;
@@ -528,6 +543,9 @@ bool KateDocumentConfig::pageUpDownMovesCursor () const
 
 void KateDocumentConfig::setPageUpDownMovesCursor (bool on)
 {
+  if (m_pageUpDownMovesCursorSet && m_pageUpDownMovesCursor == on)
+    return;
+
   configStart ();
 
   m_pageUpDownMovesCursorSet = true;
@@ -538,6 +556,9 @@ void KateDocumentConfig::setPageUpDownMovesCursor (bool on)
 
 void KateDocumentConfig::setKeepExtraSpaces(bool on)
 {
+  if (m_keepExtraSpacesSet && m_keepExtraSpaces == on)
+    return;
+
   configStart ();
 
   m_keepExtraSpacesSet = true;
@@ -556,6 +577,9 @@ bool KateDocumentConfig::keepExtraSpaces() const
 
 void KateDocumentConfig::setIndentPastedText(bool on)
 {
+  if (m_indentPastedTextSet && m_indentPastedText == on)
+    return;
+
   configStart ();
 
   m_indentPastedTextSet = true;
@@ -574,6 +598,9 @@ bool KateDocumentConfig::indentPastedText() const
 
 void KateDocumentConfig::setBackspaceIndents(bool on)
 {
+  if (m_backspaceIndentsSet && m_backspaceIndents == on)
+    return;
+
   configStart ();
 
   m_backspaceIndentsSet = true;
@@ -592,6 +619,9 @@ bool KateDocumentConfig::backspaceIndents() const
 
 void KateDocumentConfig::setSmartHome(bool on)
 {
+  if (m_smartHomeSet && m_smartHome == on)
+    return;
+
   configStart ();
 
   m_smartHomeSet = true;
@@ -610,6 +640,9 @@ bool KateDocumentConfig::smartHome() const
 
 void KateDocumentConfig::setShowTabs(bool on)
 {
+  if (m_showTabsSet && m_showTabs == on)
+    return;
+
   configStart ();
 
   m_showTabsSet = true;
@@ -628,6 +661,9 @@ bool KateDocumentConfig::showTabs() const
 
 void KateDocumentConfig::setShowSpaces(bool on)
 {
+  if (m_showSpacesSet && m_showSpaces == on)
+    return;
+
   configStart ();
 
   m_showSpacesSet = true;
@@ -646,6 +682,9 @@ bool KateDocumentConfig::showSpaces() const
 
 void KateDocumentConfig::setReplaceTabsDyn(bool on)
 {
+  if (m_replaceTabsDynSet && m_replaceTabsDyn == on)
+    return;
+
   configStart ();
 
   m_replaceTabsDynSet = true;
@@ -664,6 +703,9 @@ bool KateDocumentConfig::replaceTabsDyn() const
 
 void KateDocumentConfig::setRemoveSpaces(int triState)
 {
+  if (m_removeSpacesSet && m_removeSpaces == triState)
+    return;
+
   configStart ();
 
   m_removeSpacesSet = true;
@@ -682,6 +724,9 @@ int KateDocumentConfig::removeSpaces() const
 
 void KateDocumentConfig::setNewLineAtEof (bool on)
 {
+  if (m_newLineAtEofSet && m_newLineAtEof == on)
+    return;
+
   configStart ();
 
   m_newLineAtEofSet = true;
@@ -700,6 +745,9 @@ bool KateDocumentConfig::newLineAtEof () const
 
 void KateDocumentConfig::setOvr(bool on)
 {
+  if (m_overwiteModeSet && m_overwiteMode == on)
+    return;
+
   configStart ();
 
   m_overwiteModeSet = true;
@@ -718,6 +766,9 @@ bool KateDocumentConfig::ovr() const
 
 void KateDocumentConfig::setTabIndents(bool on)
 {
+  if (m_tabIndentsSet && m_tabIndents == on)
+    return;
+
   configStart ();
 
   m_tabIndentsSet = true;
@@ -810,6 +861,9 @@ QString KateDocumentConfig::eolString ()
 
 void KateDocumentConfig::setEol (int mode)
 {
+  if (m_eolSet && m_eol == mode)
+    return;
+
   configStart ();
 
   m_eolSet = true;
@@ -820,6 +874,9 @@ void KateDocumentConfig::setEol (int mode)
 
 void KateDocumentConfig::setBom (bool bom)
 {
+  if (m_bomSet && m_bom == bom)
+    return;
+
   configStart ();
 
   m_bomSet = true;
@@ -830,11 +887,11 @@ void KateDocumentConfig::setBom (bool bom)
 
 bool KateDocumentConfig::bom() const
 {
-    if (m_bomSet || isGlobal())
-      return m_bom;
-    return s_global->bom();
-}
+  if (m_bomSet || isGlobal())
+    return m_bom;
 
+  return s_global->bom();
+}
 
 bool KateDocumentConfig::allowEolDetection () const
 {
@@ -846,6 +903,9 @@ bool KateDocumentConfig::allowEolDetection () const
 
 void KateDocumentConfig::setAllowEolDetection (bool on)
 {
+  if (m_allowEolDetectionSet && m_allowEolDetection == on)
+    return;
+
   configStart ();
 
   m_allowEolDetectionSet = true;
@@ -854,7 +914,6 @@ void KateDocumentConfig::setAllowEolDetection (bool on)
   configEnd ();
 }
 
-
 bool KateDocumentConfig::allowSimpleMode () const
 {
   if (m_allowSimpleModeSet || isGlobal())
@@ -865,6 +924,9 @@ bool KateDocumentConfig::allowSimpleMode () const
 
 void KateDocumentConfig::setAllowSimpleMode (bool on)
 {
+  if (m_allowSimpleModeSet && m_allowSimpleMode == on)
+    return;
+
   configStart ();
 
   m_allowSimpleModeSet = true;
@@ -882,7 +944,10 @@ uint KateDocumentConfig::backupFlags () const
 }
 
 void KateDocumentConfig::setBackupFlags (uint flags)
- {
+{
+  if (m_backupFlagsSet && m_backupFlags == flags)
+    return;
+
   configStart ();
 
   m_backupFlagsSet = true;
@@ -909,6 +974,9 @@ const QString &KateDocumentConfig::backupSuffix () const
 
 void KateDocumentConfig::setBackupPrefix (const QString &prefix)
 {
+  if (m_backupPrefixSet && m_backupPrefix == prefix)
+    return;
+
   configStart ();
 
   m_backupPrefixSet = true;
@@ -919,6 +987,9 @@ void KateDocumentConfig::setBackupPrefix (const QString &prefix)
 
 void KateDocumentConfig::setBackupSuffix (const QString &suffix)
 {
+  if (m_backupSuffixSet && m_backupSuffix == suffix)
+    return;
+
   configStart ();
 
   m_backupSuffixSet = true;
@@ -937,6 +1008,9 @@ bool KateDocumentConfig::swapFileNoSync() const
 
 void KateDocumentConfig::setSwapFileNoSync(bool on)
 {
+  if (m_swapFileNoSyncSet && m_swapFileNoSync == on)
+    return;
+
   configStart();
 
   m_swapFileNoSyncSet = true;
@@ -955,6 +1029,9 @@ int KateDocumentConfig::searchDirConfigDepth () const
 
 void KateDocumentConfig::setSearchDirConfigDepth (int depth)
 {
+  if (m_searchDirConfigDepthSet && m_searchDirConfigDepth == depth)
+    return;
+
   configStart ();
 
   m_searchDirConfigDepthSet = true;
@@ -980,6 +1057,9 @@ bool KateDocumentConfig::onTheFlySpellCheck() const
 
 void KateDocumentConfig::setOnTheFlySpellCheck(bool on)
 {
+  if (m_onTheFlySpellCheckSet && m_onTheFlySpellCheck == on)
+    return;
+
   configStart ();
 
   m_onTheFlySpellCheckSet = true;
@@ -999,6 +1079,9 @@ int KateDocumentConfig::lineLengthLimit() const
 
 void KateDocumentConfig::setLineLengthLimit(int lineLengthLimit)
 {
+  if (m_lineLengthLimitSet && m_lineLengthLimit == lineLengthLimit)
+    return;
+
   configStart();
 
   m_lineLengthLimitSet = true;
@@ -1242,6 +1325,9 @@ bool KateViewConfig::dynWordWrap () const
 
 void KateViewConfig::setDynWordWrap (bool wrap)
 {
+  if (m_dynWordWrapSet && m_dynWordWrap == wrap)
+    return;
+
   configStart ();
 
   m_dynWordWrapSet = true;
@@ -1260,6 +1346,9 @@ int KateViewConfig::dynWordWrapIndicators () const
 
 void KateViewConfig::setDynWordWrapIndicators (int mode)
 {
+  if (m_dynWordWrapIndicatorsSet && m_dynWordWrapIndicators == mode)
+    return;
+
   configStart ();
 
   m_dynWordWrapIndicatorsSet = true;
@@ -1278,6 +1367,9 @@ int KateViewConfig::dynWordWrapAlignIndent () const
 
 void KateViewConfig::setDynWordWrapAlignIndent (int indent)
 {
+  if (m_dynWordWrapAlignIndentSet && m_dynWordWrapAlignIndent == indent)
+    return;
+
   configStart ();
 
   m_dynWordWrapAlignIndentSet = true;
@@ -1296,6 +1388,9 @@ bool KateViewConfig::lineNumbers () const
 
 void KateViewConfig::setLineNumbers (bool on)
 {
+  if (m_lineNumbersSet && m_lineNumbers == on)
+    return;
+
   configStart ();
 
   m_lineNumbersSet = true;
@@ -1314,6 +1409,9 @@ bool KateViewConfig::scrollBarMarks () const
 
 void KateViewConfig::setScrollBarMarks (bool on)
 {
+  if (m_scrollBarMarksSet && m_scrollBarMarks == on)
+    return;
+
   configStart ();
 
   m_scrollBarMarksSet = true;
@@ -1332,6 +1430,9 @@ bool KateViewConfig::scrollBarMiniMap () const
 
 void KateViewConfig::setScrollBarMiniMap (bool on)
 {
+  if (m_scrollBarMiniMapSet && m_scrollBarMiniMap == on)
+    return;
+
   configStart ();
 
   m_scrollBarMiniMapSet = true;
@@ -1350,6 +1451,9 @@ bool KateViewConfig::scrollBarMiniMapAll () const
 
 void KateViewConfig::setScrollBarMiniMapAll (bool on)
 {
+  if (m_scrollBarMiniMapAllSet && m_scrollBarMiniMapAll == on)
+    return;
+
   configStart ();
 
   m_scrollBarMiniMapAllSet = true;
@@ -1368,6 +1472,9 @@ int KateViewConfig::scrollBarMiniMapWidth () const
 
 void KateViewConfig::setScrollBarMiniMapWidth (int width)
 {
+  if (m_scrollBarMiniMapWidthSet && m_scrollBarMiniMapWidth == width)
+    return;
+
   configStart ();
 
   m_scrollBarMiniMapWidthSet = true;
@@ -1386,6 +1493,9 @@ bool KateViewConfig::iconBar () const
 
 void KateViewConfig::setIconBar (bool on)
 {
+  if (m_iconBarSet && m_iconBar == on)
+    return;
+
   configStart ();
 
   m_iconBarSet = true;
@@ -1404,6 +1514,9 @@ bool KateViewConfig::foldingBar () const
 
 void KateViewConfig::setFoldingBar (bool on)
 {
+  if (m_foldingBarSet && m_foldingBar == on)
+    return;
+
   configStart ();
 
   m_foldingBarSet = true;
@@ -1422,6 +1535,9 @@ bool KateViewConfig::lineModification () const
 
 void KateViewConfig::setLineModification (bool on)
 {
+  if (m_lineModificationSet && m_lineModification == on)
+    return;
+
   configStart ();
 
   m_lineModificationSet = true;
@@ -1440,6 +1556,9 @@ int KateViewConfig::bookmarkSort () const
 
 void KateViewConfig::setBookmarkSort (int mode)
 {
+  if (m_bookmarkSortSet && m_bookmarkSort == mode)
+    return;
+
   configStart ();
 
   m_bookmarkSortSet = true;
@@ -1461,6 +1580,9 @@ void KateViewConfig::setAutoCenterLines (int lines)
   if (lines < 0)
     return;
 
+  if (m_autoCenterLinesSet && m_autoCenterLines == lines)
+    return;
+
   configStart ();
 
   m_autoCenterLinesSet = true;
@@ -1478,7 +1600,10 @@ long KateViewConfig::searchFlags () const
 }
 
 void KateViewConfig::setSearchFlags (long flags)
- {
+{
+  if (m_searchFlagsSet && m_searchFlags == flags)
+    return;
+
   configStart ();
 
   m_searchFlagsSet = true;
@@ -1520,6 +1645,9 @@ uint KateViewConfig::defaultMarkType () const
 
 void KateViewConfig::setDefaultMarkType (uint type)
 {
+  if (m_defaultMarkTypeSet && m_defaultMarkType == type)
+    return;
+
   configStart ();
 
   m_defaultMarkTypeSet = true;
@@ -1548,6 +1676,9 @@ bool KateViewConfig::persistentSelection () const
 
 void KateViewConfig::setPersistentSelection (bool on)
 {
+  if (m_persistentSelectionSet && m_persistentSelection == on)
+    return;
+
   configStart ();
 
   m_persistentSelectionSet = true;
@@ -1566,19 +1697,20 @@ bool KateViewConfig::viInputMode () const
 
 void KateViewConfig::setViInputMode (bool on)
 {
-  if (!m_viInputModeSet || m_viInputMode != on) {
-    configStart ();
+  if (m_viInputModeSet && m_viInputMode == on)
+    return;
 
-    m_viInputModeSet = true;
-    m_viInputMode = on;
+  configStart ();
 
-    // make sure to turn off edits mergin when leaving vi input mode
-    if (!on && m_view) {
-      m_view->doc()->setUndoMergeAllEdits(false);
-    }
+  m_viInputModeSet = true;
+  m_viInputMode = on;
 
-    configEnd ();
+  // make sure to turn off edits mergin when leaving vi input mode
+  if (!on && m_view) {
+    m_view->doc()->setUndoMergeAllEdits(false);
   }
+
+  configEnd ();
 }
 
 bool KateViewConfig::viInputModeStealKeys () const
@@ -1591,12 +1723,13 @@ bool KateViewConfig::viInputModeStealKeys () const
 
 void KateViewConfig::setViInputModeStealKeys (bool on)
 {
-  if (!m_viInputModeStealKeysSet || m_viInputModeStealKeys != on) {
-    configStart ();
-    m_viInputModeStealKeysSet = true;
-    m_viInputModeStealKeys = on;
-    configEnd ();
-  }
+  if (m_viInputModeStealKeysSet && m_viInputModeStealKeys == on)
+    return;
+
+  configStart ();
+  m_viInputModeStealKeysSet = true;
+  m_viInputModeStealKeys = on;
+  configEnd ();
 }
 
 
@@ -1610,12 +1743,13 @@ bool KateViewConfig::viInputModeEmulateCommandBar() const
 
 void KateViewConfig::setViInputModeEmulateCommandBar(bool on)
 {
-  if (!m_viInputModeEmulateCommandBarSet || m_viInputModeEmulateCommandBar != on) {
-    configStart();
-    m_viInputModeEmulateCommandBarSet = true;
-    m_viInputModeEmulateCommandBar = on;
-    configEnd();
-  }
+  if (m_viInputModeEmulateCommandBarSet && m_viInputModeEmulateCommandBar == on)
+    return;
+
+  configStart();
+  m_viInputModeEmulateCommandBarSet = true;
+  m_viInputModeEmulateCommandBar = on;
+  configEnd();
 }
 
 bool KateViewConfig::automaticCompletionInvocation () const
@@ -1628,12 +1762,13 @@ bool KateViewConfig::automaticCompletionInvocation () const
 
 void KateViewConfig::setAutomaticCompletionInvocation (bool on)
 {
-  if (!m_automaticCompletionInvocationSet || m_automaticCompletionInvocation != on) {
-    configStart ();
-    m_automaticCompletionInvocationSet = true;
-    m_automaticCompletionInvocation = on;
-    configEnd ();
-  }
+  if (m_automaticCompletionInvocationSet && m_automaticCompletionInvocation == on)
+    return;
+
+  configStart ();
+  m_automaticCompletionInvocationSet = true;
+  m_automaticCompletionInvocation = on;
+  configEnd ();
 }
 
 bool KateViewConfig::wordCompletion () const
@@ -1646,12 +1781,13 @@ bool KateViewConfig::wordCompletion () const
 
 void KateViewConfig::setWordCompletion (bool on)
 {
-  if (!m_wordCompletionSet || m_wordCompletion != on) {
-    configStart ();
-    m_wordCompletionSet = true;
-    m_wordCompletion = on;
-    configEnd ();
-  }
+  if (m_wordCompletionSet && m_wordCompletion == on)
+    return;
+
+  configStart ();
+  m_wordCompletionSet = true;
+  m_wordCompletion = on;
+  configEnd ();
 }
 
 int KateViewConfig::wordCompletionMinimalWordLength () const
@@ -1664,6 +1800,9 @@ int KateViewConfig::wordCompletionMinimalWordLength () const
 
 void KateViewConfig::setWordCompletionMinimalWordLength (int length)
 {
+  if (m_wordCompletionMinimalWordLengthSet && m_wordCompletionMinimalWordLength == length)
+    return;
+
   configStart ();
 
   m_wordCompletionMinimalWordLengthSet = true;
@@ -1675,18 +1814,19 @@ void KateViewConfig::setWordCompletionMinimalWordLength (int length)
 bool KateViewConfig::wordCompletionRemoveTail () const
 {
   if (m_wordCompletionRemoveTailSet || isGlobal())
-  {
     return m_wordCompletionRemoveTail;
-    }
+
   return s_global->wordCompletionRemoveTail();
 }
 
 void KateViewConfig::setWordCompletionRemoveTail (bool on)
 {
+  if (m_wordCompletionRemoveTailSet && m_wordCompletionRemoveTail == on)
+    return;
+
   configStart ();
   m_wordCompletionRemoveTailSet = true;
   m_wordCompletionRemoveTail = on;
-
   configEnd ();
 }
 
@@ -1700,6 +1840,9 @@ bool KateViewConfig::smartCopyCut () const
 
 void KateViewConfig::setSmartCopyCut (bool on)
 {
+  if (m_smartCopyCutSet && m_smartCopyCut == on)
+    return;
+
   configStart ();
 
   m_smartCopyCutSet = true;
@@ -1718,6 +1861,9 @@ bool KateViewConfig::scrollPastEnd () const
 
 void KateViewConfig::setScrollPastEnd (bool on)
 {
+  if (m_scrollPastEndSet && m_scrollPastEnd == on)
+    return;
+
   configStart ();
 
   m_scrollPastEndSet = true;
@@ -1855,6 +2001,9 @@ const QString &KateRendererConfig::schema () const
 
 void KateRendererConfig::setSchema (const QString &schema)
 {
+  if (m_schemaSet && m_schema == schema)
+    return;
+
   configStart ();
   m_schemaSet = true;
   m_schema = schema;
@@ -1993,6 +2142,9 @@ const QFontMetricsF& KateRendererConfig::fontMetrics() const
 
 void KateRendererConfig::setFont(const QFont &font)
 {
+  if (m_fontSet && m_font == font)
+    return;
+
   configStart ();
 
   m_fontSet = true;
@@ -2012,6 +2164,9 @@ bool KateRendererConfig::wordWrapMarker () const
 
 void KateRendererConfig::setWordWrapMarker (bool on)
 {
+  if (m_wordWrapMarkerSet && m_wordWrapMarker == on)
+    return;
+
   configStart ();
 
   m_wordWrapMarkerSet = true;
@@ -2030,6 +2185,9 @@ const QColor& KateRendererConfig::backgroundColor() const
 
 void KateRendererConfig::setBackgroundColor (const QColor &col)
 {
+  if (m_backgroundColorSet && m_backgroundColor == col)
+    return;
+
   configStart ();
 
   m_backgroundColorSet = true;
@@ -2048,6 +2206,9 @@ const QColor& KateRendererConfig::selectionColor() const
 
 void KateRendererConfig::setSelectionColor (const QColor &col)
 {
+  if (m_selectionColorSet && m_selectionColor == col)
+    return;
+
   configStart ();
 
   m_selectionColorSet = true;
@@ -2066,6 +2227,9 @@ const QColor& KateRendererConfig::highlightedLineColor() const
 
 void KateRendererConfig::setHighlightedLineColor (const QColor &col)
 {
+  if (m_highlightedLineColorSet && m_highlightedLineColor == col)
+    return;
+
   configStart ();
 
   m_highlightedLineColorSet = true;
@@ -2096,6 +2260,10 @@ void KateRendererConfig::setLineMarkerColor (const QColor &col, KTextEditor::Mar
 {
   int index = static_cast<int>( log(static_cast<double>(type)) / log(2.0) );
   Q_ASSERT( index >= 0 && index < KTextEditor::MarkInterface::reservedMarkersCount() );
+
+  if (m_lineMarkerColorSet[index] && m_lineMarkerColor[index] == col)
+    return;
+
   configStart ();
 
   m_lineMarkerColorSet[index] = true;
@@ -2114,6 +2282,9 @@ const QColor& KateRendererConfig::highlightedBracketColor() const
 
 void KateRendererConfig::setHighlightedBracketColor (const QColor &col)
 {
+  if (m_highlightedBracketColorSet && m_highlightedBracketColor == col)
+    return;
+
   configStart ();
 
   m_highlightedBracketColorSet = true;
@@ -2132,6 +2303,9 @@ const QColor& KateRendererConfig::wordWrapMarkerColor() const
 
 void KateRendererConfig::setWordWrapMarkerColor (const QColor &col)
 {
+  if (m_wordWrapMarkerColorSet && m_wordWrapMarkerColor == col)
+    return;
+
   configStart ();
 
   m_wordWrapMarkerColorSet = true;
@@ -2150,6 +2324,9 @@ const QColor& KateRendererConfig::tabMarkerColor() const
 
 void KateRendererConfig::setTabMarkerColor (const QColor &col)
 {
+  if (m_tabMarkerColorSet && m_tabMarkerColor == col)
+    return;
+
   configStart ();
 
   m_tabMarkerColorSet = true;
@@ -2168,6 +2345,9 @@ const QColor& KateRendererConfig::indentationLineColor() const
 
 void KateRendererConfig::setIndentationLineColor (const QColor &col)
 {
+  if (m_indentationLineColorSet && m_indentationLineColor == col)
+    return;
+
   configStart ();
 
   m_indentationLineColorSet = true;
@@ -2186,6 +2366,9 @@ const QColor& KateRendererConfig::iconBarColor() const
 
 void KateRendererConfig::setIconBarColor (const QColor &col)
 {
+  if (m_iconBarColorSet && m_iconBarColor == col)
+    return;
+
   configStart ();
 
   m_iconBarColorSet = true;
@@ -2204,6 +2387,9 @@ const QColor& KateRendererConfig::foldingColor() const
 
 void KateRendererConfig::setFoldingColor (const QColor &col)
 {
+  if (m_foldingColorSet && m_foldingColor == col)
+    return;
+
   configStart ();
 
   m_foldingColorSet = true;
@@ -2254,6 +2440,9 @@ const QColor& KateRendererConfig::lineNumberColor() const
 
 void KateRendererConfig::setLineNumberColor (const QColor &col)
 {
+  if (m_lineNumberColorSet && m_lineNumberColor == col)
+    return;
+
   configStart ();
 
   m_lineNumberColorSet = true;
@@ -2272,6 +2461,9 @@ const QColor& KateRendererConfig::separatorColor() const
 
 void KateRendererConfig::setSeparatorColor(const QColor& col)
 {
+  if (m_separatorColorSet && m_separatorColor == col)
+    return;
+
   configStart ();
 
   m_separatorColorSet = true;
@@ -2290,6 +2482,9 @@ const QColor& KateRendererConfig::spellingMistakeLineColor() const
 
 void KateRendererConfig::setSpellingMistakeLineColor (const QColor &col)
 {
+  if (m_spellingMistakeLineColorSet && m_spellingMistakeLineColor == col)
+    return;
+
   configStart ();
 
   m_spellingMistakeLineColorSet = true;
@@ -2308,6 +2503,9 @@ const QColor& KateRendererConfig::modifiedLineColor() const
 
 void KateRendererConfig::setModifiedLineColor(const QColor &col)
 {
+  if (m_modifiedLineColorSet && m_modifiedLineColor == col)
+    return;
+
   configStart ();
 
   m_modifiedLineColorSet = true;
@@ -2326,6 +2524,9 @@ const QColor& KateRendererConfig::savedLineColor() const
 
 void KateRendererConfig::setSavedLineColor(const QColor &col)
 {
+  if (m_savedLineColorSet && m_savedLineColor == col)
+    return;
+
   configStart ();
 
   m_savedLineColorSet = true;
@@ -2344,6 +2545,9 @@ const QColor& KateRendererConfig::searchHighlightColor() const
 
 void KateRendererConfig::setSearchHighlightColor(const QColor &col)
 {
+  if (m_searchHighlightColorSet && m_searchHighlightColor == col)
+    return;
+
   configStart ();
 
   m_searchHighlightColorSet = true;
@@ -2362,6 +2566,9 @@ const QColor& KateRendererConfig::replaceHighlightColor() const
 
 void KateRendererConfig::setReplaceHighlightColor(const QColor &col)
 {
+  if (m_replaceHighlightColorSet && m_replaceHighlightColor == col)
+    return;
+
   configStart ();
 
   m_replaceHighlightColorSet = true;
@@ -2381,6 +2588,9 @@ bool KateRendererConfig::showIndentationLines () const
 
 void KateRendererConfig::setShowIndentationLines (bool on)
 {
+  if (m_showIndentationLinesSet && m_showIndentationLines == on)
+    return;
+
   configStart ();
 
   m_showIndentationLinesSet = true;
@@ -2399,6 +2609,9 @@ bool KateRendererConfig::showWholeBracketExpression () const
 
 void KateRendererConfig::setShowWholeBracketExpression (bool on)
 {
+  if (m_showWholeBracketExpressionSet && m_showWholeBracketExpression == on)
+    return;
+
   configStart ();
 
   m_showWholeBracketExpressionSet = true;
