@@ -2,6 +2,7 @@
  *
  *  Copyright (C) 2008 Erlend Hamberg <ehamberg@gmail.com>
  *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+ *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -32,11 +33,13 @@ class KConfigGroup;
 class KateView;
 class KateDocument;
 class KateViewInternal;
+class KateViModeBase;
 class KateViNormalMode;
 class KateViInsertMode;
 class KateViVisualMode;
 class KateViReplaceMode;
 class KateViKeyParser;
+class KateViKeyMapper;
 class QString;
 
 /**
@@ -81,11 +84,22 @@ public:
    */
   void feedKeyPresses(const QString &keyPresses) const;
 
+  /**
+   * Determines whether we are currently processing a Vi keypress
+   * @return true if we are still in a call to handleKeypress, false otherwise
+   */
+  bool isHandlingKeypress() const;
+
   /**
    * @return The current vi mode
    */
   ViMode getCurrentViMode() const;
 
+  /**
+   * @return one of getViNormalMode(), getViVisualMode(), etc, depending on getCurrentViMode().
+   */
+  KateViModeBase* getCurrentViModeHandler() const;
+
   const QString getVerbatimKeys() const;
 
   /**
@@ -136,30 +150,7 @@ public:
   /**
    * @return true if running replaying the last change due to pressing "."
    */
-  bool isReplayingLastChange() const { return m_replayingLastChange; }
-
-  /**
-   * set whether insert mode should be repeated as text instead of
-   * keystrokes. the default is keystrokes, which is reset between
-   * changes. for example, when invoking a completion command in
-   * insert mode, textual repeat should be enabled since repeating the
-   * completion may yield different results elsewhere in the document.
-   * therefore, it is better to repeat the text itself. however, most
-   * insertion commands do not need textual repeat; it should only be
-   * used when the repeating of keystrokes leads to unpredictable
-   * results. indeed, in most cases, repeating the keystrokes is the
-   * most predictable approach.
-   * @see isTextualRepeat()
-   * @return true if insert mode should be repeated as text
-   */
-  void setTextualRepeat( bool b = true ) { m_textualRepeat = b; }
-
-  /**
-   * whether insert mode should be repeated as text instead of keystrokes.
-   * @see setTextualRepeat()
-   * @return true if insert mode should be repeated as text
-   */
-  bool isTextualRepeat() const { return m_textualRepeat; }
+  bool isReplayingLastChange() const { return m_isReplayingLastChange; }
 
   /**
    * append a QKeyEvent to the key event log
@@ -169,18 +160,40 @@ public:
   /**
    * clear the key event log
    */
-  void clearLog() { m_keyEventsLog.clear(); m_keyEventsBeforeInsert.clear(); }
+  void clearCurrentChangeLog() { m_currentChangeKeyEventsLog.clear(); m_currentChangeCompletionsLog.clear(); }
 
   /**
    * copy the contents of the key events log to m_lastChange so that it can be repeated
    */
-  void storeChangeCommand();
+  void storeLastChangeCommand();
 
   /**
    * repeat last change by feeding the contents of m_lastChange to feedKeys()
    */
   void repeatLastChange();
 
+  class Completion
+  {
+  public:
+    enum CompletionType { PlainText, FunctionWithoutArgs, FunctionWithArgs };
+    Completion(const QString& completedText, bool removeTail, CompletionType completionType);
+    QString completedText() const;
+    bool removeTail() const;
+    CompletionType completionType() const;
+  private:
+    QString m_completedText;
+    bool m_removeTail;
+    CompletionType m_completionType;
+  };
+  void startRecordingMacro(QChar macroRegister);
+  void finishRecordingMacro();
+  bool isRecordingMacro();
+  void replayMacro(QChar macroRegister);
+  bool isReplayingMacro();
+  void logCompletionEvent(const Completion& completion);
+  Completion nextLoggedCompletion();
+  void doNotLogCurrentKeypress();
+
   /**
    * The current search pattern.
    * This is set by the last search.
@@ -207,8 +220,12 @@ public:
 
   void setLastSearchCaseSensitive(bool caseSensitive) { m_lastSearchCaseSensitive = caseSensitive; };
 
+  void setLastSearchPlacesCursorAtEndOfMatch(bool b) { m_lastSearchPlacedCursorAtEndOfMatch = b; };
+
   bool lastSearchCaseSensitive() { return m_lastSearchCaseSensitive; };
 
+  bool lastSearchPlacesCursorAtEndOfMatch() { return m_lastSearchPlacedCursorAtEndOfMatch; };
+
   bool getTemporaryNormalMode() { return m_temporaryNormalMode; }
 
   void setTemporaryNormalMode(bool b) {  m_temporaryNormalMode = b; }
@@ -251,6 +268,8 @@ public:
    */
   static QString modeToString(ViMode mode);
 
+  KateViKeyMapper* keyMapper();
+
 private Q_SLOTS:
   void markChanged (KTextEditor::Document* doc,
                     KTextEditor::Mark mark,
@@ -268,30 +287,46 @@ private:
   KateViewInternal *m_viewInternal;
   KateViKeyParser *m_keyParser;
 
+  // Create a new keymapper for each macro event, to simplify expansion of mappings in macros
+  // where the macro itself was triggered by expanding a mapping!
+  QStack<QSharedPointer<KateViKeyMapper> > m_keyMapperStack;
+
+  int m_insideHandlingKeyPressCount;
+
   /**
    * set to true when replaying the last change (due to e.g. pressing ".")
    */
-  bool m_replayingLastChange;
+  bool m_isReplayingLastChange;
+
+  bool m_isRecordingMacro;
+
+  QChar m_recordingMacroRegister;
+  QList<QKeyEvent> m_currentMacroKeyEventsLog;
+
+  int m_macrosBeingReplayedCount;
+  QChar m_lastPlayedMacroRegister;
+
+  QList<Completion> m_currentMacroCompletionsLog;
 
   /**
-   * set to true when the insertion should be repeated as text
+   * Stuff for retrieving the next completion for the macro.
+   * Needs to be on a stack for if macros call other macros
+   * which have their own stored completions.
    */
-  bool m_textualRepeat;
+  QStack<QList<Completion> > m_macroCompletionsToReplay;
+  QStack< int > m_nextLoggedMacroCompletionIndex;
 
   /**
    * a continually updated list of the key events that was part of the last change.
    * updated until copied to m_lastChange when the change is completed.
    */
-  QList<QKeyEvent> m_keyEventsLog;
+  QList<QKeyEvent> m_currentChangeKeyEventsLog;
+  QList<Completion> m_currentChangeCompletionsLog;
+  QList<Completion> m_lastChangeCompletionsLog;
+  int m_nextLoggedLastChangeComplexIndex;
 
   /**
-   * like m_keyEventsLog, but excludes insert mode: that is, only the key events
-   * leading up to insert mode (like "i", "cw", and "o") are logged.
-   */
-  QList<QKeyEvent> m_keyEventsBeforeInsert;
-
-  /**
-   * a list of the key events that was part of the last change.
+   * a list of the (encoded) key events that was part of the last change.
    */
   QString m_lastChange;
 
@@ -305,6 +340,8 @@ private:
    */
   bool m_lastSearchCaseSensitive;
 
+  bool m_lastSearchPlacedCursorAtEndOfMatch;
+
   /**
    * true when normal mode was started by Ctrl-O command in insert mode.
    */
@@ -313,7 +350,7 @@ private:
   /**
    * true when mark set inside viinputmodemanager to do not serve it as bookmark set;
    */
-  bool m_mark_set_inside_viinputmodemanager;
+  bool m_markSetInsideViInputModeManager;
 
   // jump list
   QList<KateViJump> *jump_list;
