@@ -2,6 +2,7 @@
  *
  *  Copyright (C) 2008-2011 Erlend Hamberg <ehamberg@gmail.com>
  *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+ *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -43,6 +44,7 @@ enum BlockInsert {
 
 class KATEPART_TESTS_EXPORT KateViInsertMode : public KateViModeBase
 {
+  Q_OBJECT
   public:
     KateViInsertMode( KateViInputModeManager *viInputModeManager, KateView * view, KateViewInternal * viewInternal );
     ~KateViInsertMode();
@@ -71,12 +73,6 @@ class KATEPART_TESTS_EXPORT KateViInsertMode : public KateViModeBase
     bool commandInsertContentOfRegister();
     bool commandSwitchToNormalModeForJustOneCommand();
 
-    // mappings not supported in insert mode yet
-    void addMapping( const QString &from, const QString &to, MappingRecursion recursion ) { Q_UNUSED(from) Q_UNUSED(to) Q_UNUSED(recursion)}
-    const QString getMapping( const QString &from ) const { Q_UNUSED(from) return QString(); }
-    const QStringList getMappings() const { return QStringList(); }
-    bool isMappingRecursive(const QString& from) const { Q_UNUSED(from) return false; };
-
     void setBlockPrependMode( KateViRange blockRange );
     void setBlockAppendMode( KateViRange blockRange, BlockInsert b );
 
@@ -94,7 +90,17 @@ class KATEPART_TESTS_EXPORT KateViInsertMode : public KateViModeBase
     unsigned int m_count;
     bool m_countedRepeatsBeginOnNewLine;
 
+    bool m_isExecutingCompletion;
+    QString m_textInsertedByCompletion;
+    Cursor m_textInsertedByCompletionEndPos;
+
     void leaveInsertMode( bool force = false);
+
+    void completionFinished();
+    void replayCompletion();
+    int findNextMergeableBracketPos(const Cursor& startPos);
+  private slots:
+    void textInserted(KTextEditor::Document* document, KTextEditor::Range range);
 };
 
 #endif
