@@ -3,6 +3,7 @@
  *  Copyright (C) 2008 - 2012 Erlend Hamberg <ehamberg@gmail.com>
  *  Copyright (C) 2009 Paul Gideon Dann <pdgiddie@gmail.com>
  *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+ *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -189,6 +190,11 @@ Range KateViModeBase::findPattern(const QString& pattern, bool backwards, bool c
   Cursor searchBegin = startFrom;
   KTextEditor::Search::SearchOptions flags = KTextEditor::Search::Regex;
 
+  if (count == -1)
+  {
+    count = getCount();
+  }
+
   if ( backwards ) {
     flags |= KTextEditor::Search::Backwards;
   }
@@ -305,7 +311,7 @@ KateViRange KateViModeBase::findPatternForMotion( const QString& pattern, bool b
 
   Range match = findPattern(pattern, backwards, caseSensitive, startFrom, count);
 
-  return KateViRange( match.start().line(), match.start().column(), ViMotion::ExclusiveMotion );
+  return KateViRange( match.start().line(), match.start().column(), match.end().line(), match.end().column(), ViMotion::ExclusiveMotion );
 }
 
 Cursor KateViModeBase::findNextWordStart( int fromLine, int fromColumn, bool onlyCurrentLine ) const
@@ -335,10 +341,10 @@ Cursor KateViModeBase::findNextWordStart( int fromLine, int fromColumn, bool onl
 
     if ( c1 == -1 && c2 == -1 && c3 == -1 ) {
         if ( onlyCurrentLine ) {
-            return Cursor( l, c );
+            return Cursor::invalid();
         } else if ( l >= doc()->lines()-1 ) {
             c = qMax(line.length()-1, 0);
-            return Cursor( l, c );
+            return Cursor::invalid();
         } else {
             c = 0;
             l++;
@@ -383,7 +389,7 @@ Cursor KateViModeBase::findNextWORDStart( int fromLine, int fromColumn, bool onl
   QRegExp startOfWORD("\\s\\S");
 
   while ( !found ) {
-    c = startOfWORD.indexIn( line, c+1 );
+    c = startOfWORD.indexIn( line, c );
 
     if ( c == -1 ) {
       if ( onlyCurrentLine ) {
@@ -437,15 +443,14 @@ Cursor KateViModeBase::findPrevWordEnd( int fromLine, int fromColumn, bool onlyC
           c = c1;
       } else {
           if ( onlyCurrentLine ) {
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else if ( l > 0 ) {
               line = getLine( --l );
               c = line.length();
 
               continue;
           } else {
-              c = 0;
-              return Cursor( l, c );
+              return Cursor::invalid();
           }
       }
   }
@@ -474,7 +479,7 @@ Cursor KateViModeBase::findPrevWORDEnd( int fromLine, int fromColumn, bool onlyC
           c = c1;
       } else {
           if ( onlyCurrentLine ) {
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else if ( l > 0 ) {
               line = getLine( --l );
               c = line.length();
@@ -482,7 +487,7 @@ Cursor KateViModeBase::findPrevWORDEnd( int fromLine, int fromColumn, bool onlyC
               continue;
           } else {
               c = 0;
-              return Cursor( l, c );
+              return Cursor::invalid();
           }
       }
   }
@@ -519,9 +524,9 @@ Cursor KateViModeBase::findPrevWordStart( int fromLine, int fromColumn, bool onl
 
     if ( c1 == -1 && c2 == -1 && c3 == -1 && c4 == -1 ) {
       if ( onlyCurrentLine ) {
-          return Cursor( l, c );
+          return Cursor::invalid();
       } else if ( l <= 0 ) {
-        return Cursor( 0, 0 );
+        return Cursor::invalid();
       } else {
         line = getLine( --l );
         c = line.length();
@@ -572,9 +577,9 @@ Cursor KateViModeBase::findPrevWORDStart( int fromLine, int fromColumn, bool onl
 
     if ( c1 == -1 && c2 == -1 ) {
       if ( onlyCurrentLine ) {
-          return Cursor( l, c );
+          return Cursor::invalid();
       } else if ( l <= 0 ) {
-        return Cursor( 0, 0 );
+        return Cursor::invalid();
       } else {
         line = getLine( --l );
         c = line.length();
@@ -626,10 +631,10 @@ Cursor KateViModeBase::findWordEnd( int fromLine, int fromColumn, bool onlyCurre
           c = c1;
       } else {
           if ( onlyCurrentLine ) {
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else if ( l >= doc()->lines()-1 ) {
               c = line.length()-1;
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else {
               c = -1;
               line = getLine( ++l );
@@ -661,10 +666,10 @@ Cursor KateViModeBase::findWORDEnd( int fromLine, int fromColumn, bool onlyCurre
           c = c1;
       } else {
           if ( onlyCurrentLine ) {
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else if ( l >= doc()->lines()-1 ) {
               c = line.length()-1;
-              return Cursor( l, c );
+              return Cursor::invalid();
           } else {
               c = -1;
               line = getLine( ++l );
@@ -685,7 +690,7 @@ KateViRange innerRange(KateViRange range, bool inner) {
     if ((r.startLine == r.endLine) && columnDistance == 1 )
     {
       // Start and end are right next to each other; there is nothing inside them.
-      r.valid = false;
+      return KateViRange::invalid();
     }
     r.startColumn++;
     r.endColumn--;
@@ -746,8 +751,7 @@ KateViRange KateViModeBase::findSurroundingQuotes( const QChar &c, bool inner )
     }
 
     // Nothing found - give up :)
-     r.valid = false;
-     return r;
+    return KateViRange::invalid();
   }
 
 
@@ -755,7 +759,7 @@ KateViRange KateViModeBase::findSurroundingQuotes( const QChar &c, bool inner )
   r.endColumn = line.indexOf( c, cursor.column() );
 
   if ( r.startColumn == -1 || r.endColumn == -1 || r.startColumn > r.endColumn ) {
-    r.valid = false;
+    return KateViRange::invalid();
   }
 
   return innerRange(r, inner);
@@ -815,8 +819,7 @@ KateViRange KateViModeBase::findSurroundingBrackets( const QChar &c1,
     }
 
     if (!should_break) {
-      r.valid = false;
-      return r;
+      return KateViRange::invalid();
     }
 
     r.endColumn = column;
@@ -865,8 +868,7 @@ KateViRange KateViModeBase::findSurroundingBrackets( const QChar &c1,
     }
 
     if (!should_break) {
-      r.valid = false;
-      return r;
+      return KateViRange::invalid();
     }
 
     r.startColumn = column;
@@ -888,7 +890,7 @@ KateViRange KateViModeBase::findSurrounding( const QRegExp &c1, const QRegExp &c
   KateViRange r( cursor.line(), col1, cursor.line(), col2, ViMotion::InclusiveMotion );
 
   if ( col1 == -1 || col2 == -1 || col1 > col2 ) {
-      r.valid = false;
+    return KateViRange::invalid();
   }
 
   if ( inner ) {
@@ -1058,63 +1060,114 @@ KateViRange KateViModeBase::goVisualLineUpDown(int lines) {
   KateViRange r( c.line(), c.column(), ViMotion::InclusiveMotion );
   int tabstop = doc()->config()->tabWidth();
 
-  // if in an empty document, just return
   if ( lines == 0 ) {
+    // We're not moving anywhere.
     return r;
   }
 
-  int line_height = m_view->renderer()->lineHeight();
-
-  QPoint point = m_view->cursorToCoordinate(c);
-  Cursor res = m_view->coordinatesToCursor(QPoint(point.x(), point.y() + lines * line_height));
-
-  r.endColumn = res.column();
-  r.endLine = res.line();
-
-  if (r.endColumn == -1 || r.endLine == -1 || r.valid == false)
-    return r;
-
-  if ( m_stickyColumn == -1) {
-    // Compute new sticky column.
-    const Kate::TextLine startLine = doc()->plainKateTextLine( c.line() );
-    const int virtColumnStart = startLine->toVirtualColumn(c.column(), tabstop);
-    const int visualColumnStart = virtColumnStart - m_viewInternal->cache()->textLayout(c).startCol();
-    //Q_ASSERT(m_stickyColumn >= 0);
-    m_stickyColumn = visualColumnStart;
-    Q_ASSERT(m_stickyColumn >= 0);
-  } else {
-    const int endViewLine = m_viewInternal->cache()->viewLine(res);
-    // The "real" (non-virtual) beginning of the current "line", which might be a wrapped continuation of a
-    // "real" line.
-    const int realLineStartColumn = m_viewInternal->cache()->textLayout(res.line(), endViewLine).startCol();
-    const Kate::TextLine endLine = doc()->plainKateTextLine( r.endLine );
-    // Adjust for the fact that if the portion of the line before wrapping is indented,
-    // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
-    const bool isWrappedContinuation = (m_viewInternal->cache()->textLayout(res.line(), endViewLine).lineLayout().lineNumber() != 0);
-    const int numInvisibleIndentChars = isWrappedContinuation ? endLine->toVirtualColumn(m_viewInternal->cache()->line(res.line())->textLine()->nextNonSpaceChar(0), tabstop) : 0;
-    if (m_stickyColumn == (unsigned int)KateVi::EOL)
+  // Work out the real and visual line pair of the beginning of the visual line we'd end up
+  // on by moving lines visual lines.  We ignore the column, for now.
+  int finishVisualLine = m_viewInternal->cache()->viewLine(m_view->cursorPosition());
+  int finishRealLine = m_view->cursorPosition().line();
+  int count = qAbs(lines);
+  bool invalidPos = false;
+  if (lines > 0)
+  {
+    // Find the beginning of the visual line "lines" visual lines down.
+    while (count > 0)
     {
-      const int visualEndColumn = m_viewInternal->cache()->textLayout(res.line(), endViewLine).lineLayout().textLength() - 1;
-      r.endColumn = endLine->fromVirtualColumn( visualEndColumn + realLineStartColumn - numInvisibleIndentChars, tabstop );
-    }
-    else
-    {
-      // Algorithm: find the "real" column corresponding to the start of the line.  Offset from that
-      // until the "visual" column is equal to the "visual" sticky column.
-      int realOffsetToVisualStickyColumn = 0;
-      const int lineStartVirtualColumn = endLine->toVirtualColumn( realLineStartColumn, tabstop );
-      while (true)
+      finishVisualLine++;
+      if (finishVisualLine >= m_viewInternal->cache()->line(finishRealLine)->viewLineCount())
       {
-        const int visualColumn = endLine->toVirtualColumn( realLineStartColumn + realOffsetToVisualStickyColumn, tabstop ) - lineStartVirtualColumn + numInvisibleIndentChars;
-        if (visualColumn >= m_stickyColumn)
+        finishRealLine++;
+        finishVisualLine = 0;
+      }
+      if (finishRealLine >= doc()->lines())
+      {
+        invalidPos = true;
+        break;
+      }
+      count--;
+    }
+  }
+  else
+  {
+    // Find the beginning of the visual line "lines" visual lines up.
+    while (count > 0)
+    {
+      finishVisualLine--;
+      if (finishVisualLine < 0)
+      {
+        finishRealLine--;
+        if (finishRealLine < 0)
         {
+          invalidPos = true;
           break;
         }
-        realOffsetToVisualStickyColumn++;
+        finishVisualLine = m_viewInternal->cache()->line(finishRealLine)->viewLineCount() - 1;
       }
-      r.endColumn = realLineStartColumn + realOffsetToVisualStickyColumn;
+      count--;
     }
   }
+  if (invalidPos)
+  {
+    r.endLine = -1;
+    r.endColumn = -1;
+    return r;
+  }
+
+  // We know the final (real) line ...
+  r.endLine = finishRealLine;
+  // ... now work out the final (real) column.
+
+  if ( m_stickyColumn == -1 || !m_lastMotionWasVisualLineUpOrDown) {
+    // Compute new sticky column. It is a *visual* sticky column.
+    int startVisualLine = m_viewInternal->cache()->viewLine(m_view->cursorPosition());
+    int startRealLine = m_view->cursorPosition().line();
+    const Kate::TextLine startLine = doc()->plainKateTextLine( c.line() );
+    // Adjust for the fact that if the portion of the line before wrapping is indented,
+    // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
+    const bool isWrappedContinuation = (m_viewInternal->cache()->textLayout(startRealLine, startVisualLine).lineLayout().lineNumber() != 0);
+    const int numInvisibleIndentChars = isWrappedContinuation ? startLine->toVirtualColumn(m_viewInternal->cache()->line(startRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+
+    const int realLineStartColumn = m_viewInternal->cache()->textLayout(startRealLine, startVisualLine).startCol();
+    const int lineStartVirtualColumn = startLine->toVirtualColumn( realLineStartColumn, tabstop );
+    const int visualColumnNoInvisibleIndent  = startLine->toVirtualColumn(c.column(), tabstop) - lineStartVirtualColumn;
+    m_stickyColumn = visualColumnNoInvisibleIndent + numInvisibleIndentChars;
+    Q_ASSERT(m_stickyColumn >= 0);
+  }
+
+  // The "real" (non-virtual) beginning of the current "line", which might be a wrapped continuation of a
+  // "real" line.
+  const int realLineStartColumn = m_viewInternal->cache()->textLayout(finishRealLine, finishVisualLine).startCol();
+  const Kate::TextLine endLine = doc()->plainKateTextLine( r.endLine );
+  // Adjust for the fact that if the portion of the line before wrapping is indented,
+  // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
+  const bool isWrappedContinuation = (m_viewInternal->cache()->textLayout(finishRealLine, finishVisualLine).lineLayout().lineNumber() != 0);
+  const int numInvisibleIndentChars = isWrappedContinuation ? endLine->toVirtualColumn(m_viewInternal->cache()->line(finishRealLine)->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+  if (m_stickyColumn == (unsigned int)KateVi::EOL)
+  {
+    const int visualEndColumn = m_viewInternal->cache()->textLayout(finishRealLine, finishVisualLine).lineLayout().textLength() - 1;
+    r.endColumn = endLine->fromVirtualColumn( visualEndColumn + realLineStartColumn - numInvisibleIndentChars, tabstop );
+  }
+  else
+  {
+    // Algorithm: find the "real" column corresponding to the start of the line.  Offset from that
+    // until the "visual" column is equal to the "visual" sticky column.
+    int realOffsetToVisualStickyColumn = 0;
+    const int lineStartVirtualColumn = endLine->toVirtualColumn( realLineStartColumn, tabstop );
+    while (true)
+    {
+      const int visualColumn = endLine->toVirtualColumn( realLineStartColumn + realOffsetToVisualStickyColumn, tabstop ) - lineStartVirtualColumn + numInvisibleIndentChars;
+      if (visualColumn >= m_stickyColumn)
+      {
+        break;
+      }
+      realOffsetToVisualStickyColumn++;
+    }
+    r.endColumn = realLineStartColumn + realOffsetToVisualStickyColumn;
+  }
+  m_currentMotionWasVisualLineUpOrDown = true;
 
   return r;
 }
@@ -1125,8 +1178,8 @@ bool KateViModeBase::startNormalMode()
   // store the key presses for this "insert mode session" so that it can be repeated with the
   // '.' command
   if (!m_viInputModeManager->isReplayingLastChange()) {
-    m_viInputModeManager->storeChangeCommand();
-    m_viInputModeManager->clearLog();
+    m_viInputModeManager->storeLastChangeCommand();
+    m_viInputModeManager->clearCurrentChangeLog();
   }
 
   m_viInputModeManager->viEnterNormalMode();
@@ -1279,43 +1332,67 @@ void KateViModeBase::addToNumberUnderCursor( int count )
       return;
     }
 
-    int wordStart = findPrevWordStart( c.line(), c.column()+1, true ).column();
-
-    if (wordStart > 0 && line.at(wordStart - 1) == '-') wordStart--;
-
-    QRegExp number( "(0x)([0-9a-fA-F]+)|\\-?\\d+" );
-
-    int start = number.indexIn( line, wordStart );
-
-    QString nString = number.cap();
-    bool ok = false;
-    int base = number.cap( 1 ).isEmpty() ? 10 : 16;
-    if (base != 16 && nString.startsWith("0") && nString != "0")
+    int numberStartPos = -1;
+    QString numberAsString;
+    QRegExp numberRegex( "(0x)([0-9a-fA-F]+)|\\-?\\d+" );
+    const int cursorColumn = c.column();
+    const int currentLineLength = doc()->lineLength(c.line());
+    const Cursor prevWordStart = findPrevWordStart(c.line(), cursorColumn);
+    int wordStartPos = prevWordStart.column();
+    if (prevWordStart.line() < c.line())
     {
-      nString.toInt( &ok, 8 );
+      // The previous word starts on the previous line: ignore.
+      wordStartPos = 0;
+    }
+    if (wordStartPos > 0 && line.at(wordStartPos - 1) == '-') wordStartPos--;
+    for (int searchFromColumn = wordStartPos; searchFromColumn < currentLineLength; searchFromColumn++)
+    {
+      numberStartPos = numberRegex.indexIn( line, searchFromColumn );
+
+      numberAsString = numberRegex.cap();
+
+      if (numberStartPos + numberAsString.length() <= c.column())
+      {
+        // This number is before the cursor; keep on searching.
+        numberStartPos = -1;
+      }
+      else
+      {
+        // This'll do!
+        break;
+      }
+    }
+
+    if (numberStartPos == -1)
+    {
+      // None found.
+      return;
+    }
+
+    bool ok = false;
+    int base = numberRegex.cap( 1 ).isEmpty() ? 10 : 16;
+    if (base != 16 && numberAsString.startsWith("0") && numberAsString != "0")
+    {
+      numberAsString.toInt( &ok, 8 );
       if (ok)
       {
         // Octal.
         base = 8;
       }
     }
-    int n = nString.toInt( &ok, base );
-    QString withoutBase;
+    const int originalNumber = numberAsString.toInt( &ok, base );
+    QString withoutBase = numberAsString;
     if (base == 16)
     {
-      withoutBase = number.cap(2);
+      withoutBase = numberRegex.cap(2);
     }
     else if (base == 8)
     {
-      withoutBase = nString.mid(1); // Strip off leading 0.
-    }
-    else
-    {
-      withoutBase = nString;
+      withoutBase = numberAsString.mid(1); // Strip off leading 0.
     }
 
     kDebug( 13070 ) << "base: " << base;
-    kDebug( 13070 ) << "n: " << n;
+    kDebug( 13070 ) << "n: " << originalNumber;
 
     if ( !ok ) {
         // conversion to int failed. give up.
@@ -1323,27 +1400,27 @@ void KateViModeBase::addToNumberUnderCursor( int count )
     }
 
     // increase/decrease number
-    n += count;
+    const int newNumber = originalNumber + count;
 
-    // create the new text string to be inserted. prepend with “0x” if in base 16, and "0" if base 8.
+    // Create the new text string to be inserted. Prepend with “0x” if in base 16, and "0" if base 8.
     // For non-decimal numbers, try to keep the length of the number the same (including leading 0's).
-    QString newNumberPadded = (base == 16 || base == 8) ? QString("%1").arg(n, withoutBase.length(), base, QChar('0')) : QString("%1").arg(n, 0, base);
-    QString newText = newNumberPadded;
+    QString newNumberPadded = (base == 16 || base == 8) ? QString("%1").arg(newNumber, withoutBase.length(), base, QChar('0')) : QString("%1").arg(newNumber, 0, base);
+    QString newNumberText = newNumberPadded;
     if (base == 16)
     {
-      newText = "0x" + newText;
+      newNumberText = "0x" + newNumberText;
     }
     else if (base == 8)
     {
-      newText = "0" + newText;
+      newNumberText = "0" + newNumberText;
     }
 
-    // replace the old number string with the new
+    // Replace the old number string with the new.
     doc()->editStart();
-    doc()->removeText( KTextEditor::Range( c.line(), start , c.line(), start+nString.length() ) );
-    doc()->insertText( KTextEditor::Cursor( c.line(), start ), newText );
+    doc()->removeText( KTextEditor::Range( c.line(), numberStartPos , c.line(), numberStartPos+numberAsString.length() ) );
+    doc()->insertText( KTextEditor::Cursor( c.line(), numberStartPos ), newNumberText );
     doc()->editEnd();
-    updateCursor(Cursor(m_view->cursorPosition().line(), start + newText.length() - 1));
+    updateCursor(Cursor(m_view->cursorPosition().line(), numberStartPos + newNumberText.length() - 1));
 }
 
 void KateViModeBase::switchView(Direction direction) {
@@ -1363,7 +1440,7 @@ void KateViModeBase::switchView(Direction direction) {
   int curr_cursor_x = m_view->mapToGlobal(m_view->cursorToCoordinate(m_view->cursorPosition())).x();
 
   KateView *bestview = NULL;
-  int  best_x1, best_x2, best_y1, best_y2, best_center_y, best_center_x;
+  int  best_x1 = -1, best_x2 = -1, best_y1 = -1, best_y2 = -1, best_center_y = -1, best_center_x = -1;
 
   if (direction == Next && visible_views.count() != 1) {
     for (int i=0; i< visible_views.count(); i++) {
