@@ -3,6 +3,7 @@
  *  Copyright (C) 2008 - 2009 Erlend Hamberg <ehamberg@gmail.com>
  *  Copyright (C) 2009 Paul Gideon Dann <pdgiddie@gmail.com>
  *  Copyright (C) 2011 Svyatoslav Kuzmich <svatoslav1@gmail.com>
+ *  Copyright (C) 2012 - 2013 Simon St James <kdedevel@etotheipiplusone.com>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -49,9 +50,6 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
 {
   Q_OBJECT
 
-  public slots:
-    void mappingTimerTimeOut();
-
   public:
     KateViNormalMode( KateViInputModeManager *viInputModeManager, KateView * view, KateViewInternal * viewInternal );
     virtual ~KateViNormalMode();
@@ -81,6 +79,7 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     bool commandMakeUppercase();
     bool commandMakeUppercaseLine();
     bool commandChangeCase();
+    bool commandChangeCaseRange();
 
     bool commandOpenNewLineUnder();
     bool commandOpenNewLineOver();
@@ -170,6 +169,9 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     bool commandExpandLocal();
     bool commandToggleRegionVisibility();
 
+    bool commandStartRecordingMacro();
+    bool commandReplayMacro();
+
     // MOTIONS
 
     KateViRange motionLeft();
@@ -235,6 +237,8 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     KateViRange motionToBeforeParagraph();
     KateViRange motionToAfterParagraph();
 
+    KateViRange motionToIncrementalSearchMatch();
+
     // TEXT OBJECTS
 
     KateViRange textObjectAWord();
@@ -268,14 +272,8 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
 
     void addCurrentPositionToJumpList();
 
-    void addMapping( const QString& from, const QString& to, KateViModeBase::MappingRecursion recursion );
-    const QString getMapping( const QString &from ) const;
-    const QStringList getMappings() const;
-    bool isMappingRecursive(const QString& from) const;
     virtual void reset();
 
-    void setMappingTimeout(int timeoutMS);
-
     void beginMonitoringDocumentChanges();
   protected:
     void resetParser();
@@ -314,8 +312,8 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
 
     bool m_linewiseCommand;
     bool m_commandWithMotion;
-    bool m_isCommandBeingExecuted;
     bool m_lastMotionWasLinewiseInnerBlock;
+    bool m_motionCanChangeWholeVisualModeSelection;
 
     bool m_commandShouldKeepSelection;
 
@@ -331,21 +329,6 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     QHash<QString, QString> m_matchingItems;
     QRegExp m_matchItemRegex;
 
-    // mappings
-    bool m_mappingKeyPress;
-    // Will be the mapping used if we decide that no extra mapping characters will be
-    // typed, either because we have a mapping that cannot be extended to another
-    // mapping by adding additional characters, or we have a mapping and timed out waiting
-    // for it to be extended to a longer mapping.
-    // (Essentially, this allows us to have mappings that extend each other e.g. "'12" and
-    // "'123", and to choose between them.)
-    QString m_fullMappingMatch;
-    // set after f/F/t/T/r so the following character isn't translated
-    bool m_doNotMapNextKeyPress;
-    QString m_mappingKeys;
-    bool m_doNotExpandFurtherMappings;
-    void executeMapping();
-
     KateViKeyParser *m_keyParser;
 
     // Ctrl-c or ESC have been pressed, leading to a call to reset().
@@ -360,6 +343,10 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     Cursor m_currentChangeEndMarker;
 
     bool m_isUndo;
+
+    Cursor m_positionWhenIncrementalSearchBegan;
+
+    bool waitingForRegisterOrCharToSearch();
 private slots:
     void textInserted(KTextEditor::Document* document, KTextEditor::Range range);
     void textRemoved(KTextEditor::Document*,KTextEditor::Range);
